import QtQuick 2.15
import QtQuick.Window 2.15
//import QtQuick.Controls 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.4
import QtQuick.Controls.Styles 1.4

import "./notetreeview"
import "./titlebar"
import "./notelistview"
import "./notewebview"

import NoteBacked 1.0
import org.deepin.dtk 1.0

ApplicationWindow {
    id: mainWindow
    visible: true
    minimumWidth: 678
    minimumHeight: 300
    width: 1100
    height: 800

    property int currentMainWindowShowModel: InteractionManager.ThreeColumn
    property int m_iSelectedNoteSum: 1
    property var m_multiSelectArray:[]

    // Add window properties
    DWindow.enabled: true
    flags: {Qt.Window | Qt.WindowMinMaxButtonsHint | Qt.WindowCloseButtonHint | Qt.WindowTitleHint}
    header: TitleBar {
        height:50
        Titlebar {
            id: titlebar
        }
    }

    /* 监控主窗口宽度变化 */
    onWidthChanged: { mainWindow.mainWindowWidthResize() }

    Rectangle {
        id: main
        anchors.fill: parent

        SplitView {
            id: spliView
            anchors.fill: parent
            orientation: Qt.Horizontal

            /* 分割线设置 */
            handleDelegate: Rectangle {
                width: 1.5
                color: isLightTheme() ? "#F8F8F8" : "#181818"
                opacity: 1
            }

            /* 笔记本tree视图 */
            Rectangle {
                id: leftView
                property alias noteTreeAlias: noteTree
                implicitWidth: 210
                Layout.minimumWidth: 193
                Layout.maximumWidth: 300

                onWidthChanged: {
                    if (leftView.width <= 153) {
                        main.mainViewSwitchShowModel(InteractionManager.TwoColumn)
                        titlebar.setViewMode(InteractionManager.TwoColumn)
                    }
                }

                LeftTreeView {
                    id: noteTree
                }
            }

            /* 笔记list列表视图 */
            Rectangle {
                id: middleSplitView
                implicitWidth: 310
                Layout.minimumWidth: 180
                Layout.maximumWidth: 514
                visible: true

                NoteListView {
                    id: noteListView
                }
            }

            /* 笔记web编辑区视图 */
            Rectangle {
                id: rightView
                visible: true
                Layout.minimumWidth: 498
                implicitWidth: 580

                RichTextEditor {
                    id: noteWebView
                    anchors.fill: rightView
                    visible: true
                }

                property alias defaultPageAlias: defaultPage
                DefaultPage {
                    id: defaultPage
                    anchors.centerIn: rightView
                    visible: false
                }

                property alias multiSelectViewAlias: multiSelectView
                MultiSelectView {
                    id: multiSelectView
                    anchors.centerIn: rightView
                    visible: false
                }
            }
        }

        Component.onCompleted: {
            //排序菜单切换通知消息绑定
            leftView.noteTreeAlias.signalNoticeMenuStyle.connect(titlebar.setFolderType);

            /* 绑定缺省页面显示信号 */
            noteListView.sigShowNotelistWebView.connect(showNotelistWebView);

            /* 绑定缺省页面显示信号(标题栏) */
            noteListView.sigShowNotelistWebView.connect(titlebar.setDefaultPage);

            /* 笔记列表里的笔记数量变化信号 */
            noteListView.sigNoteAmountChanged.connect(leftView.noteTreeAlias.noticeUpdateTreeData);

            /* 绑定界面视图模式切换信号 */
            titlebar.mainViewSwitchSignal.connect(mainViewSwitchShowModel)

            //编辑区发生数据变动通知LISTVIEW
            noteWebView.jsBackend.sigNoteContentChange.connect(webViewChangeData);
            noteWebView.jsBackend.sigNoteNameChange.connect(webViewChangeData);

            //标题栏显示或者隐藏编辑工具栏
            titlebar.showCommonBarSignal.connect(function(bShow){
                noteWebView.jsBackend.slotShowToolbar(bShow)
            })

            //标题栏插入图片
            titlebar.insertImageSignal.connect(function(listDatas){
                noteWebView.jsBackend.slotInsertImages(listDatas)
                noteWebView.editor.focus = true
            })
            mainWindow.x = mainWindow.screen.virtualX + mainWindow.screen.width / 2 - mainWindow.width / 2;
            mainWindow.y = mainWindow.screen.virtualY + mainWindow.screen.height / 2 - mainWindow.height / 2;

            stateManager.sigActiveWindow.connect(function(){
                mainWindow.requestActivate();
                mainWindow.show();
            });
            titlebar.startRecordSignal.connect(function(){noteWebView.audioManager.startRecorder()});
            titlebar.searchSignal.connect(search)
        }

        //编辑区数据发生变动
        function webViewChangeData(ret) {
            var index = noteListView.getcurrentindex();
            noteListView.listdata.onDataUpdate(index);
            var iRet = noteListView.listdata.checkNeedSort(index);
            noteListView.setNewIndex(iRet);
        }
        
        /* 搜索 */
        function search(key, type) {
            /* 笔记列表显示 */
            noteListView.search(key, type)
            /* web页面显示 */
            noteWebView.searchKey = key
            noteWebView.editor.findText(key)
        }

        /* 主视图显示模式切换（三栏/双栏/专注） */
        function mainViewSwitchShowModel(iMainViewShowModel) {
            switch (iMainViewShowModel) {
                /* 三栏模式 */
            case InteractionManager.ThreeColumn:
                leftView.visible = true
                if (defaultPage.visible === false) {
                    middleSplitView.visible = true
                }
                rightView.visible = true
                leftView.width = 194
                var middleSplitViewTemp = middleSplitView.width
                var iTreeViewNeedWidth = leftView.width + middleSplitView.width + rightView.width
                var iWidthTemp = iTreeViewNeedWidth - mainWindow.width
                if (iWidthTemp > 0) {
                    mainWindow.width = iTreeViewNeedWidth
                    middleSplitView.width = middleSplitViewTemp
                }
                currentMainWindowShowModel = InteractionManager.ThreeColumn
                break;

                /* 双栏模式 */
            case InteractionManager.TwoColumn:
                leftView.visible = false
                if (defaultPage.visible === false) {
                    middleSplitView.visible = true
                }
                rightView.visible = true
                currentMainWindowShowModel = InteractionManager.TwoColumn
                break;

                /* 专注模式 */
            case InteractionManager.FocusColumn:
                leftView.visible = false
                middleSplitView.visible = false
                rightView.visible = true
                currentMainWindowShowModel = InteractionManager.FocusColumn
                break;

            default: break;
            }
        }

        /* 显示NoteList笔记列表视图 和 noteWeb笔记web视图，或者显示缺省视图 */
        function showNotelistWebView(isNeedShowDefaultView, isRecycle) {

            defaultPage.m_isRecycle = isRecycle
            if (isNeedShowDefaultView) {
                defaultPage.visible     = true
                noteWebView.visible     = false
                middleSplitView.visible = false
            } else {
                if (middleSplitView.visible === false) {
                    if (currentMainWindowShowModel != InteractionManager.FocusColumn) {
                        middleSplitView.visible = true
                    }
                }

                if (noteWebView.visible === false) {
                    noteWebView.jsBackend.sigCallJsSetReadonly(isRecycle)
                    noteWebView.visible = true
                }

                defaultPage.visible = false
            }
        }
    }

    /* 快捷键 */
    Item {
        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+N"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("Ctrl+N")
                if (!stateManager.getState(StatesManager.StateRecording)
                        || stateManager.getState(StatesManager.StatePlaying)
                        || stateManager.getState(StatesManager.StateVoice2Text)
                        || stateManager.getState(StatesManager.StateSearching)) {
                    noteTree.creatNoteBookDialog(InteractionManager.CreatNotebook, "")
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "F2"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info(stateManager.getState(StatesManager.StateSearching))
                if (!noteTree.isRenamed && !stateManager.getState(StatesManager.StateSearching)) {
                    noteTree.shortCutRename()
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "delete"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("delete")

                var state = !stateManager.getState(StatesManager.StateRecording)
                        && !stateManager.getState(StatesManager.StatePlaying)
                        && !stateManager.getState(StatesManager.StateVoice2Text)

                if (noteTree.activeFocus && state) {
                    noteTree.deleteNoteBookDialog()
                } else if (noteListView.activeFocus && state && noteListView.m_abstractListAlias.getListCount()){
                    if (noteTree.getFolderType() === InteractionManager.RECYCLEROOT) {
                        noteListView.deleteNoteConfirm()
                    } else {
                        noteListView.deleteNote()
                    }
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "Space"
            context: Qt.ApplicationShortcut
            onActivated: {
                if (stateManager.getState(StatesManager.StateRecording)) {
                    noteWebView.audioManager.startRecorder();
                }
                if(stateManager.getState(StatesManager.StatePlaying)){
                    noteWebView.audioManager.startPlay("", false)
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+R"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("Ctrl+R")
                if (!defaultPage.visible) {
                    titlebar.startRecordSignal()
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+B"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("Ctrl+B")
                if (noteTree.getFolderType() !== InteractionManager.RECYCLEROOT) {
                    noteListView.onDefaultPageAddNoteSig()
                }
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+F"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("Ctrl+F")
                titlebar.startSearchSignal()
            }
        }

        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+Y"
            context: Qt.ApplicationShortcut
            onActivated: {console.info("Ctrl+Y")}
        }

        Shortcut {
            autoRepeat: false
            sequence: "F1"
            context: Qt.ApplicationShortcut
            onActivated: {console.info("F1")}
        }

        Shortcut {
            autoRepeat: false
            sequence: "Ctrl+Shift+/"
            context: Qt.ApplicationShortcut
            onActivated: {
                console.info("Ctrl+Shift+/")
                interactionManager.previewShortcut(Qt.point(x + (width / 2), y + (height / 2)))
            }
        }
    }


    /* 主窗口宽度变化响应接口 */
    function mainWindowWidthResize() {
        if (rightView.width <= 498) {
            if (leftView.visible === true) {
                middleSplitView.width = mainWindow.width - rightView.width - leftView.width
            } else {
                middleSplitView.width = mainWindow.width - rightView.width
            }
        }

        if (rightView.width <= 498 && middleSplitView.width <= 180) {
            leftView.width = mainWindow.width - rightView.width - middleSplitView.width

            if (leftView.width === 193) {
                main.mainViewSwitchShowModel(InteractionManager.TwoColumn)
                titlebar.setViewMode(InteractionManager.TwoColumn)
            }
        }
    }

    /* 判断应用是否是浅色主题 */
    function isLightTheme() {
        var bRet = false

        if (DTK.themeType === 1) {
            bRet = true
        }

        return bRet
    }
}

