import QtQuick 2.0
import QtQuick.Layouts 1.11
import NoteBacked 1.0
import org.deepin.dtk 1.0
import org.deepin.dtk.style 1.0 as DS

FloatingPanel {
    leftPadding: 10
    rightPadding: 10
    topPadding: 0
    bottomPadding: 0
    id: pane
    property bool pause: false
    RowLayout{
        anchors.fill: parent
        DciIcon {
            id: playerbtn
            name: pane.pause ? "pause" : "play";
            sourceSize: Qt.size(32, 32)
            palette: DTK.makeIconPalette(pane.palette)
            mode: pane.ColorSelector.controlState
            theme: DTK.themeType
            Layout.leftMargin: 20
            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton
                propagateComposedEvents: true
                onClicked: {
                    startPlay("", false)
                }
            }
        }
        ColumnLayout {
            spacing : 0
            Layout.leftMargin: 20
            Layout.rightMargin: 20
            AudioPlayManager {
                id: playerControl
                Layout.preferredHeight: 34
                Layout.fillWidth: true
                Component.onCompleted: {
                    playerControl.sigPlayFinsh.connect(function(){
                        pane.pause = false
                    })
                }
            }
            RowLayout{
                 spacing : 10
                 Label {
                     Layout.preferredHeight: 17
                     Layout.fillWidth: true
                     Layout.alignment: Qt.AlignLeft
                     wrapMode: Text.WrapAnywhere
                     elide: Text.ElideRight
                     text: playerControl.audioTitle
                     horizontalAlignment: Qt.AlignLeft
                     font: DTK.fontManager.t5
                     color: DS.Style.control.selectColor(palette.window, Qt.rgba(0, 0, 0, 1.0), Qt.rgba(255, 255, 255, 1.0))
                 }
                 Label {
                     Layout.preferredHeight: 15
                     Layout.minimumWidth: 56
                     Layout.alignment: Qt.AlignRight
                     wrapMode: Text.WrapAnywhere
                     text: playerControl.playTime
                     horizontalAlignment: Qt.AlignRight
                     font: DTK.fontManager.t6
                     color: DS.Style.control.selectColor(palette.window, Qt.rgba(0, 0, 0, 0.6), Qt.rgba(255, 255, 255, 0.6))
                 }
            }
        }

        ActionButton {
            icon.name: "entry_clear"
            Layout.rightMargin: 20
            height: 16
            width: 16
            onClicked: {
                colseWindow();
            }
        }
    }

    function startPlay(data, isReset){
        if(isReset){
            pane.pause = false;
        }
        var ret = playerControl.slotStartPlay(data, isReset);
        if(0 === ret){
            pane.pause = !pane.pause;
            return true;
        }
        return false
    }

    function stopPlay(){
        playerControl.slotPlayFinsh();
    }

    function colseWindow(){
        stopPlay();
        audioManager.visible = false
    }
}

