import QtQuick 2.0
import QtQuick.Layouts 1.11
import NoteBacked 1.0
import org.deepin.dtk 1.0
import org.deepin.dtk.style 1.0 as DS

FloatingPanel {
    leftPadding: 10
    rightPadding: 10
    topPadding: 0
    bottomPadding: 0
    id: pane
    property bool pause: false
    property alias audioControl: recorderControl

    RowLayout{
        anchors.fill: parent

        DciIcon {
            id: playerbtn
            name: pane.pause ? "recording_pause" : "recording";
            sourceSize: Qt.size(32, 32)
            palette: DTK.makeIconPalette(pane.palette)
            mode: pane.ColorSelector.controlState
            theme: DTK.themeType
            Layout.leftMargin: 20
            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton
                propagateComposedEvents: true
                onClicked: {
                    startPauseRecord()
                }
            }
        }

        AudioRecordManager {
            id: recorderControl
            Layout.preferredHeight: 36
            Layout.fillWidth: true
            Layout.leftMargin: 10
        }

        DciIcon {
            name:  "stop"
            sourceSize: Qt.size(32, 32)
            mode: pane.ColorSelector.controlState
            theme: DTK.themeType
            Layout.leftMargin: 15
            Layout.rightMargin: 10
            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton
                propagateComposedEvents: true
                onClicked: {
                   stopRecord()
                }
            }
        }

        Label {
            Layout.minimumWidth: 65
            Layout.alignment: Qt.AlignRight
            wrapMode: Text.WrapAnywhere
            font: DTK.fontManager.t3
            text: recorderControl.recordTime
            horizontalAlignment: Qt.AlignCenter
            color: DS.Style.control.selectColor(palette.window, Qt.rgba(0, 0, 0, 1.0), Qt.rgba(255, 255, 255, 1.0))
        }
        ActionButton {
            icon.name: "entry_clear"
            Layout.rightMargin: 20
            Layout.leftMargin: 10
            height: 16
            width: 16
            onClicked: {
                colseWindow();
            }
        }
    }

    function startPauseRecord(){
        if(!pane.pause){
            if(!recorderControl.slotStartRecord()){
                return false;
            }
        }else{
            recorderControl.slotPauseRecord();
        }
        pane.pause = !pane.pause
        return true
    }

    function stopRecord(){
        pane.pause = false;
        recorderControl.slotStopRecord();
    }

    function colseWindow(){
        stopRecord();
        audioManager.visible = false
    }
}
