import QtQuick 2.9
import QtQuick.Window 2.2
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.11
import QtGraphicalEffects 1.13

import org.deepin.dtk 1.0
import NoteBacked 1.0
import "."

Rectangle {
    id: abstractListRoot
    readonly property int itemDelegateHeitht: 40
    readonly property int itemDelegateMiniWidth: 204

    property alias listViewAlias: listView

    width: middleSplitView.width

    MouseArea {
        id: absRectMouseArea
        anchors.fill: parent
        acceptedButtons: Qt.RightButton
        propagateComposedEvents: true
        onClicked: {
            if (mouse.button & Qt.RightButton) {
                var point = mapToItem(noteListViewRoot, mouseX, mouseY)
                noteListViewRoot.noteListBlankMenuAlias.x = point.x
                noteListViewRoot.noteListBlankMenuAlias.y = point.y
                noteListViewRoot.noteListBlankMenuAlias.open()
            }
        }
    }

    ListView {
        id: listView
        anchors.margins: 10
        anchors.left: parent.left
        anchors.right: parent.right
        implicitHeight: contentHeight
        height: abstractListRoot.height
        model: modellist
        spacing: 10
        clip: true /* 当元素的子项超出父项范围后会自动裁剪 */
        boundsBehavior: Flickable.StopAtBounds /* 取消鼠标拖拽拉伸效果 */

        /* 按住shift，连续多选的起点 */
        property int mulBegin: 0
        /* 单选信号 */
        signal checkOne(int idx)
        /* 按shift多选信号 */
        signal checkMul(int idx)

        /* 设置指定item项高亮 */
        function setListItemCheck(iIndex) {
            m_bAutoExclusive = true
            if (iIndex >= 0) {
                listView.currentIndex = iIndex
                listView.itemAtIndex(listView.currentIndex).checked = true
            }
        }

        /* 清除清楚列表所有的高亮item项 */
        function clearListItemChecked() {
            for (var i = 0; i <= listView.count - 1; i++) {
                listView.itemAtIndex(i).checked = false
            }
        }

        /* 更新多选高亮数组 */
        function updateMultiSelectArray() {
            m_multiSelectArray = []
            for (var i = 0; i < listView.count; i++) {
                if (listView.itemAtIndex(i).checked) {
                    m_multiSelectArray.push(i)
                }
            }
            m_iSelectedNoteSum = m_multiSelectArray.length
        }

        /* 高亮保存在数组里的index item项 */
        function highlightMultiItem() {
            clearListItemChecked()
            if (m_iSelectedNoteSum < 2) {
                return
            }

            for (var i = 0; i < m_multiSelectArray.length; i++) {
                listView.itemAtIndex(m_multiSelectArray[i]).checked = true
            }
         }

        Connections {
            target: listView

            function onCheckOne(idx) {
                listView.mulBegin = idx
            }
        }

        ScrollBar.vertical: ScrollBar {
            id: asbListScrollBar
        }

        delegate: ItemDelegate {
           id: noteItemDelegate
           width:  abstractListRoot.width - 20
           height: 108
           checked: index === 0
           autoExclusive: m_bAutoExclusive
           //checked: index === ListView.view.count - 1
           //backgroundVisible: index % 2 === 0
           backgroundVisible: true
           onCheckedChanged: {
               if (checked) {
                   noteListViewRoot.m_currentIndex = index
                   /* 获取笔记id，将id值传递给Web视图 */
                   noteWebView.currentNoteId = IdRole
                   noteListViewRoot.listdata.resetDoSort();
               }
           }

           contentFlow: true
           content:
               ColumnLayout{
               spacing: 10
                RowLayout {
                   id: rowContent
                   spacing: 0

                   Column {
                       spacing: 8

                       Row{
                           spacing: 2

                           DciIcon {
                               name: "sticky_notes"
                               sourceSize {
                                   width: 20
                                   height: 20
                               }
                               visible: TopRole ? true : false
                           }

                           Label {
                               id: abstractTitleLabel
                               elide: Text.ElideRight;
                               width: SynImgRole ? noteItemDelegate.width - 20 - noteThumbnail.width : noteItemDelegate.width - 10
                               font: DTK.fontManager.t6
                               color: isLightTheme() ? (checked ? Qt.rgba(255/255, 255/255, 255/255, 1) : Qt.rgba(0/255, 0/255, 0/255, 0.9)) : (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.9))

                               text: NameRole
                           }
                       }

                       Label {
                           id: noteAbstractLabel
                           font: DTK.fontManager.t8
                           wrapMode: Text.WrapAnywhere
                           width: SynImgRole ? noteItemDelegate.width - 20 - noteThumbnail.width : noteItemDelegate.width - 10
                           height: 36
                           elide: Text.ElideRight
                           Layout.fillWidth: true
                           Layout.fillHeight: true
                           color: isLightTheme() ? (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.6)) : (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.6))

                           text: SynContentRole
                       }

                       Label {
                           elide: Text.ElideRight;
                           width: SynImgRole ? noteItemDelegate.width - 20 - noteThumbnail.width : noteItemDelegate.width - 10
                           font: DTK.fontManager.t9
                           color: isLightTheme() ? (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.6) : Qt.rgba(0/255, 0/255, 0/255, 0.6)) : (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.6))

                           text: DisplayTimeRole
                       }
                   }

                   ColumnLayout {
                       MouseArea {
                           width: 60
                           height: 60
                           enabled: false

                           Image {
                               id: noteThumbnail
                               anchors.fill: parent
                               visible: false
                               source: SynImgRole
                           }

                           Rectangle {
                               id: imageRectangle
                               anchors.fill: parent
                               radius: 8
                               visible: false
                           }

                           OpacityMask {
                               anchors.fill: parent
                               source: noteThumbnail
                               maskSource: imageRectangle
                           }
                       }
                    }
                }

                MenuSeparator {
                    visible: TopLineRole ? true : false
                }
           }

           Connections {
               target: listView

               function onCheckOne(idx) {
                   noteItemDelegate.checked = (idx === index)
               }

               function onCheckMul(idx) {
                   if (idx > listView.mulBegin) {
                       noteItemDelegate.checked = (index >= listView.mulBegin && index <= idx)
                   } else {
                       noteItemDelegate.checked = (index <= listView.mulBegin && index >= idx)
                   }
               }
           }

           MouseArea {
               id: listItemMouse
               anchors.fill: parent
               hoverEnabled: true
               acceptedButtons: Qt.RightButton | Qt.LeftButton
               onClicked: {
                   noteListViewRoot.forceActiveFocus()
                   //listView.currentIndex = index
                   //noteItemDelegate.checked = true
                   if (mouse.button & Qt.RightButton) {
                       var point =  mapToItem(noteListViewRoot, mouseX, mouseY)
                       if (noteItemDelegate.checked === true && m_iSelectedNoteSum >= 2) {
                           noteListViewRoot.noteListMulSelectMenuAlias.x = point.x
                           noteListViewRoot.noteListMulSelectMenuAlias.y = point.y
                           noteListViewRoot.noteListMulSelectMenuAlias.open()
                       } else {
                           listRightMenu.x = point.x
                           listRightMenu.y = point.y
                           listRightMenu.open()
                       }
                   }

                   switch (mouse.modifiers) {
                   case Qt.ControlModifier:
                       if ((mouse.button & Qt.RightButton)) {
                           listRightMenu.close()
                       }
                       m_bAutoExclusive = false
                       noteItemDelegate.checked = !noteItemDelegate.checked
                       listView.updateMultiSelectArray();
                       noteListViewRoot.statiSelectedNotes(index, noteItemDelegate.checked)
                       break;

                   case Qt.ShiftModifier:
                       m_bAutoExclusive = false
                       listView.checkMul(index);
                       listView.updateMultiSelectArray()
                       noteListViewRoot.statiSelectedNotes(index, noteItemDelegate.checked)
                       break;

                   default:
                       if ((mouse.button & Qt.RightButton) && noteItemDelegate.checked === true && m_iSelectedNoteSum >= 2) {
                           return
                       }
                       m_bAutoExclusive = true
                       listView.checkOne(index);
                       listView.mulBegin = index;
                       m_multiSelectArray = []
                       m_iSelectedNoteSum = 1
                       rightView.multiSelectViewAlias.visible = false
                       break;
                   }
               }
           }
        }

        MouseArea {
            id: absRectMouseArea2
            anchors.fill: parent
            acceptedButtons: Qt.LeftButton
            propagateComposedEvents: true
            onClicked: {
                listView.focus = true
                mouse.accepted = false
            }
        }
    }
}
