import QtQuick 2.4
import QtQuick.Layouts 1.11
import QtQuick.Window 2.11
import QtQuick.Controls 2.4
import org.deepin.dtk 1.0
import NoteBacked 1.0

DialogWindow {
    id: dialog

    width: 400
    minimumWidth: 400
    maximumWidth: 400
    icon: "deepin-voice-note"
    modality: Qt.WindowModal

    property int result: Dialog.Rejected
    /* 多选数目 */
    property int multipleNumber
    property string query
    property string remind
    property string rejectText: "Cancel"
    property string acceptText: "Delete"

    /* 确认信号 */
    signal sigAccepted()

    /* 销毁弹窗 */
    signal sigDestroy()

    ColumnLayout {
        width: parent.width
        Label {
            id:quer
            Layout.maximumWidth: 360
            Layout.alignment: Qt.AlignHCenter
            font: DTK.fontManager.t5
            wrapMode: Text.WrapAnywhere
            text: qsTr(query)
        }
        Label {
            Layout.alignment: Qt.AlignHCenter
            font: DTK.fontManager.t7
            text: qsTr(remind)
        }
        RowLayout {
            Layout.alignment: Qt.AlignBottom | Qt.AlignHCenter
            Layout.bottomMargin: 10
            Layout.topMargin: 10
            Layout.fillWidth: true

            Button {
                id: cancelBt
                text: qsTr(rejectText)
                Layout.preferredWidth: 175
                font: DTK.fontManager.t6
                onClicked: done(Dialog.Rejected)
                Keys.onReturnPressed: done(Dialog.Rejected)
                Keys.onEscapePressed: done(Dialog.Rejected)
            }

            Item {Layout.fillWidth: true}

            WarningButton {
                text: qsTr(acceptText)
                Layout.preferredWidth: 175
                font: DTK.fontManager.t6
                Layout.alignment: Qt.AlignRight
                onClicked: done(Dialog.Accepted)
                Keys.onEscapePressed: done(Dialog.Rejected)
                Keys.onReturnPressed: done(Dialog.Accepted)
            }
        }
    }

    function setDialogType(type) {
        switch(type) {
        case(InteractionManager.DeleteNotebook):
            query = "Are you sure you want to delete this notebook?"
            remind = "All notes inside the notebook will be moved to the Trash"
            break
        case(InteractionManager.PermanentlyDeleteNotes):
            query = "Are you sure you want to permanently delete these notes?"
            remind = "You cannot recover them later"
            break
        case(InteractionManager.MultipleDeleteNotes):
            query = "Are you sure you want to permanently delete these" + multipleNumber + "notes?"
            remind = "You cannot recover them later"
            break
        case(InteractionManager.ClearRecycle):
            query = "Are you sure you want to empty the Trash?"
            remind = "You cannot recover the notes in it later"
            acceptText = "Empty"
            break
        }
    }

    function done(type) {
        result = type
        close()
    }


    onClosing:{
        if (result === Dialog.Accepted) {
            dialog.sigAccepted()
        }
        dialog.sigDestroy()
        close.accepted = true;
    }

    onActiveChanged: {
        cancelBt.forceActiveFocus(Qt.TabFocusReason)
    }
}




