import QtQuick 2.9
import QtQuick.Window 2.2
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.11

import org.deepin.dtk 1.0 as D
import org.deepin.dtk 1.0
import NoteBacked 1.0
import "."

Rectangle{
    id : datalistRoot
    width: middleSplitView.width

    property alias listViewAlias: listView

    MouseArea {
        id: absRectMouseArea
        anchors.fill: parent
        acceptedButtons: Qt.RightButton
        propagateComposedEvents: true
        onClicked: {
            if (mouse.button & Qt.RightButton) {
                var point = mapToItem(noteListViewRoot, mouseX, mouseY)
                noteListViewRoot.noteListBlankMenuAlias.x = point.x
                noteListViewRoot.noteListBlankMenuAlias.y = point.y
                noteListViewRoot.noteListBlankMenuAlias.open()
            }
        }
    }

    ListView {
        id: listView
        anchors.margins: 10
        anchors.left: parent.left
        anchors.right: parent.right
        implicitHeight: contentHeight
        height: datalistRoot.height
        model: modellist
        spacing: 8
        clip: true
        boundsBehavior: Flickable.StopAtBounds /* 取消鼠标拖拽拉伸效果 */

        /* 按住shift，连续多选的起点 */
        property int mulBegin: 0
        /* 单选信号 */
        signal checkOne(int idx)
        /* 按shift多选信号 */
        signal checkMul(int idx)

        Connections {
            target: listView

            function onCheckOne(idx) {
                listView.mulBegin = idx
            }
        }

        ScrollBar.vertical: ScrollBar {
            id: dataListScrollBar
        }

        /* 设置指定item项高亮 */
        function setListItemCheck(iIndex) {
            if (iIndex >= 0) {
                listView.currentIndex = iIndex
                listView.itemAtIndex(listView.currentIndex).checked = true
            }
        }

        /* 清除清楚列表所有的高亮item项 */
        function clearListItemChecked() {
            for (var i = 0; i <= listView.count - 1; i++) {
                listView.itemAtIndex(i).checked = false
            }
        }

        /* 更新多选高亮数组 */
        function updateMultiSelectArray() {
            m_multiSelectArray = []
            for (var i = 0; i < listView.count; i++) {
                if (listView.itemAtIndex(i).checked) {
                    m_multiSelectArray.push(i)
                }

                m_iSelectedNoteSum = m_multiSelectArray.length
            }
        }

        /* 高亮保存在数组里的index item项 */
        function highlightMultiItem() {
            clearListItemChecked()
            //clearListItemChecked()
            if (m_iSelectedNoteSum < 2) {
                return
            }

            clearListItemChecked()
            for (var i = 0; i < m_multiSelectArray.length; i++) {
                listView.itemAtIndex(m_multiSelectArray[i]).checked = true
            }
         }

        delegate: ItemDelegate {
           id: noteItemDelegate
           width: datalistRoot.width - 20
           height: 36
           checked: index === 0
           //checked: index === ListView.view.count - 1
           //backgroundVisible: index % 2 === 0
           autoExclusive: m_bAutoExclusive
           backgroundVisible: true
           onCheckedChanged: {
               if (checked) {
                   noteListViewRoot.m_currentIndex = index
                   /* 获取笔记id，将id值传递给Web视图 */
                   noteWebView.currentNoteId = IdRole
                   noteListViewRoot.listdata.resetDoSort();
               }
           }

           contentFlow: true
           content:ColumnLayout{
               spacing: 7
                   RowLayout {
                   id: rowContent
                   width: noteItemDelegate.width
                   spacing: 5

                   DciIcon {
                       id: datalistTop
                       name: "sticky_notes"
                       sourceSize {
                           width: 20
                           height: 20
                       }
                       visible: TopRole ? true : false
                   }

                   Row {
                       Label {
                           id: dataListTitleLabel
                           width: noteItemDelegate.width - dataListTimeLabel.width - 20 - datalistTop.width
                           height: 20
                           Layout.maximumHeight: 20
                           wrapMode: Text.WrapAnywhere
                           elide: Text.ElideRight
                           verticalAlignment: Text.AlignVCenter
                           font: DTK.fontManager.t6
                           color: isLightTheme() ? (checked ? Qt.rgba(255/255, 255/255, 255/255, 1) : Qt.rgba(0/255, 0/255, 0/255, 0.9)) : (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.9))

                           text: NameRole
                       }
                   }

                   Label {
                       id: dataListTimeLabel
                       width: 110
                       Layout.maximumHeight: 20
                       wrapMode: Text.WrapAnywhere
                       elide: Text.ElideRight
                       horizontalAlignment: Text.AlignLeft
                       font: DTK.fontManager.t9
                       color: isLightTheme() ? (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.6) : Qt.rgba(0/255, 0/255, 0/255, 0.6)) : (checked ? Qt.rgba(255/255, 255/255, 255/255, 0.9) : Qt.rgba(0/255, 0/255, 0/255, 0.6))

                       text: DisplayTimeRole
                   }
               }

               MenuSeparator{
                   visible: TopLineRole ? true : false
               }
           }

           Connections {
               target: listView

               function onCheckOne(idx) {
                   noteItemDelegate.checked = (idx === index)
               }

               function onCheckMul(idx) {
                   if (idx > listView.mulBegin) {
                       noteItemDelegate.checked = (index >= listView.mulBegin && index <= idx)
                   } else {
                       noteItemDelegate.checked = (index <= listView.mulBegin && index >= idx)
                   }
               }
           }

           MouseArea {
               id: listItemMouse
               anchors.fill: parent
               hoverEnabled: true
               acceptedButtons: Qt.RightButton | Qt.LeftButton
               onClicked: {
                   //listView.currentIndex = index
                   //noteItemDelegate.checked = true
                   if (mouse.button & Qt.RightButton) {
                       var point =  mapToItem(noteListViewRoot, mouseX, mouseY)
                       if (noteItemDelegate.checked === true && m_iSelectedNoteSum >= 2) {
                           noteListViewRoot.noteListMulSelectMenuAlias.x = point.x
                           noteListViewRoot.noteListMulSelectMenuAlias.y = point.y
                           noteListViewRoot.noteListMulSelectMenuAlias.open()
                       } else {
                           listRightMenu.x = point.x
                           listRightMenu.y = point.y
                           listRightMenu.open()
                       }
                   }

                   switch (mouse.modifiers) {
                   case Qt.ControlModifier:
                       if ((mouse.button & Qt.RightButton)) {
                           listRightMenu.close()
                       }
                       m_bAutoExclusive = false
                       noteItemDelegate.checked = !noteItemDelegate.checked
                       listView.updateMultiSelectArray()
                       noteListViewRoot.statiSelectedNotes(noteItemDelegate.checked)
                       break;

                   case Qt.ShiftModifier:
                       m_bAutoExclusive = false
                       listView.checkMul(index);
                       listView.updateMultiSelectArray()
                       noteListViewRoot.statiSelectedNotes(index, noteItemDelegate.checked)
                       break;

                   default:
                       if ((mouse.button & Qt.RightButton) && noteItemDelegate.checked === true && m_iSelectedNoteSum >= 2) {
                           return
                       }
                       m_bAutoExclusive = true
                       listView.checkOne(index);
                       listView.mulBegin = index;
                       m_multiSelectArray = []
                       m_iSelectedNoteSum = 1
                       rightView.multiSelectViewAlias.visible = false
                       break;
                   }
               }
           }
        }

        MouseArea {
            id: absRectMouseArea2
            anchors.fill: parent
            acceptedButtons: Qt.LeftButton
            propagateComposedEvents: true
            onClicked: {
                listView.focus = true
                mouse.accepted = false
            }
        }
    }
}
