import QtQuick 2.0
import QtQuick.Controls 2.4
import QtQuick.Window 2.11
import QtQuick.Layouts 1.11
import QtGraphicalEffects 1.12
import NoteBacked 1.0

import org.deepin.dtk 1.0 as D
import org.deepin.dtk 1.0

import "./../notetreeview"
import "./../titlebar"
import "./../"

Rectangle {
    id: noteListViewRoot
    width: middleSplitView.width
    height: middleSplitView.height

    DataListModel {
        id: modellist
    }

    property alias listdata: modellist
    property int m_folderContentId: 0
    property string m_noteBookName: ""
    property int m_currentIndex: 0
    property int m_selectFileType: InteractionManager.AbstractView
    property alias m_dataListAlias: m_dataList
    property alias m_abstractListAlias: m_abstractList
    property alias noteListBlankMenuAlias: noteListBlankMenu
    property alias noteListMulSelectMenuAlias: noteListMulSelectMenu
    property bool m_bAutoExclusive: true
    property var m_multiSelectArray:[]

    /* 显示缺省页面信号 */
    signal sigShowNotelistWebView(bool isNeedShowDefaultView, bool isRecycle)
    /* 笔记列表里的笔记数量变化信号 */
    signal sigNoteAmountChanged()

    function getcurrentindex() {
        if (m_selectFileType === InteractionManager.AbstractView){
            return noteListViewRoot.m_abstractListAlias.listViewAlias.currentIndex;
        } else if (m_selectFileType === InteractionManager.CardView) {
            return noteListViewRoot.m_cardList.listViewAlias.currentIndex;
        } else if (m_selectFileType === InteractionManager.ListView) {
            return noteListViewRoot.m_dataListAlias.listViewAlias.currentIndex;
        }
    }

    Label {
        id: noteListTitle
        width: middleSplitView.width - 20
        height: 29
        anchors.top: noteListViewRoot.top
        anchors.left: noteListViewRoot.left
        anchors.topMargin: 10
        anchors.leftMargin: 10
        color: isLightTheme() ? Qt.rgba(0, 0, 0, 1) : Qt.rgba(0, 0, 0, 0.9)
        elide: Text.ElideRight
        font: DTK.fontManager.t4

        text: qsTr(m_noteBookName)
    }

    Text {
        id: topText
        anchors.left: noteListViewRoot.left
        anchors.leftMargin: 10
        anchors.top: noteListTitle.bottom
        anchors.topMargin: 5
        visible : false
        text: qsTr("Sticky Notes")
    }

    DataList {
        id : m_dataList
        visible: false;
        anchors.top: topText.bottom
        height: middleSplitView.height - noteListTitle.height - 40
        anchors.bottom: noteListViewRoot.bottom
    }

    AbstractList {
        id : m_abstractList
        visible: true;
        height: middleSplitView.height - noteListTitle.height - 40
        anchors.bottom: noteListViewRoot.bottom
    }

    CardList {
        id : m_cardList
        visible: false;
        anchors.bottom: noteListViewRoot.bottom
    }

    ListRightMenu {
        id: listRightMenu
    }

    Loader {
        id: popup
        property int dialogType
        onLoaded: {
            item.setDialogType(dialogType)
            item.sigDestroy.connect(noteListViewRoot.popupDestory)
        }
    }

    Menu {
        id: noteListBlankMenu

        MenuItem {
            text: noteListViewRoot.isRecycleroot() ? qsTr("Empty Trash") : qsTr("New note")
            height: visible ? implicitHeight : 0
            onClicked: {
                if (noteListViewRoot.isRecycleroot()) {
                    noteListViewRoot.clearRecycleConfirm()
                } else {
                    noteListViewRoot.addNote()
                }
            }
        }
    }

    Menu {
        id: noteListMulSelectMenu

        MenuItem {
            text: qsTr("New note")
            height: visible ? implicitHeight : 0
            onClicked: {

            }
        }

        MenuItem {
            text: qsTr("Move note")
            height: visible ? implicitHeight : 0
            onClicked: {

            }
        }

        Menu {
            title: qsTr("Export")

            MenuItem {
                text: qsTr("Export html")
                height: visible ? implicitHeight : 0
                onClicked: {

                }
            }

            MenuItem {
                text: qsTr("Export pdf")
                height: visible ? implicitHeight : 0
                onClicked: {

                }
            }
        }

        MenuItem {
            text: qsTr("Remove")
            height: visible ? implicitHeight : 0
            onClicked: {

            }
        }
    }

    Rectangle {
            y: noteListViewRoot.height - 68
            height: 68
            width: parent.width
            color: "transparent"

            LinearGradient {
                id: linearGradient
                anchors.fill: parent
                start: Qt.point(width / 2, 0)
                end: Qt.point(width / 2, height)
                gradient: Gradient {
                    GradientStop { position: 0.0; color: Qt.rgba(255,255,255,0.00) }
                    GradientStop { position: 0.88; color: "#ffffff" }
                }
                opacity: 0.8
            }

            MouseArea {
                anchors.fill: parent
	         hoverEnabled: true
            }

            FloatingButton {
                id: addNoteButton
                anchors.centerIn: parent
                width: 49
                height: 48
                anchors.horizontalCenter: noteListViewRoot.horizontalCenter
                checked: true
                visible: isRecycleroot() ? false : true

                icon.name: "action_add"
                icon.width: 29
                icon.height: 28

                onClicked: {
                    noteListViewRoot.addNote();
                }
            }
        }

    /* 信号链接并绑定对应函数 */
    Component.onCompleted: {
        /* 笔记列表视图展示模式信号绑定 */
        titlebar.listViewSwitchSignal.connect(listViewSwitchShowModel)

        /* 笔记本点击-笔记列表展示联动信号绑定 */
        leftView.noteTreeAlias.sigShowNoteList.connect(loadNoteList)

        /* 笔记本右键“新建笔记”菜单项信号绑定 */
        leftView.noteTreeAlias.signalAddNote.connect(treeViewMenuAddNote)

        /* 绑定缺省页面新建笔记信号 */
        rightView.defaultPageAlias.sigDefaultPageAddNote.connect(onDefaultPageAddNoteSig)

        /*清空回收站操作*/
        leftView.noteTreeAlias.signalDeleteAll.connect(clearRecycleConfirm)

        /*笔记排序功能*/
        titlebar.listViewSortSignal.connect(listNoteSort)
    }

    /* 笔记列表切换显示模式 */
    function listViewSwitchShowModel(iNoteListViewModel) {

        switch (iNoteListViewModel) {
            /* 列表显示模式 */
        case InteractionManager.ListView:
            m_dataList.visible = true
            m_abstractList.visible  = false
            m_cardList.visible = false
            m_selectFileType = InteractionManager.ListView
            m_dataList.listViewAlias.highlightMultiItem()
            if (m_iSelectedNoteSum < 2) {
                m_dataList.listViewAlias.setListItemCheck(m_currentIndex)
            }
            break;

            /* 摘要显示模式 */
        case InteractionManager.AbstractView:
            m_dataList.visible = false
            m_abstractList.visible  = true
            m_cardList.visible = false
            m_selectFileType = InteractionManager.AbstractView;
            m_abstractList.listViewAlias.highlightMultiItem()
            if (m_iSelectedNoteSum < 2) {
                m_abstractList.listViewAlias.setListItemCheck(m_currentIndex)
            }
            break;

            /* 卡片显示模式 */
        case InteractionManager.CardView:
            m_dataList.visible = false
            m_abstractList.visible  = false
            m_cardList.visible = true
            m_selectFileType = InteractionManager.CardView;
            break;

        default: break;
        }
    }

    /* 点击笔记本展示联动展现对应笔记列表，并更新笔记本名称 */
    function loadNoteList(folderContentId, noteBookName) {
        m_noteBookName = noteBookName

        m_folderContentId = folderContentId;
        if (folderContentId !== 1) {
            noteListTitle.text = m_noteBookName;
        }
        editItemSource()
    }

    /* 编辑数据源 */
    function editItemSource() {
        var isNeedShowDefaultView = false
        var bRet = modellist.editItemSource(m_folderContentId, getcurrentindex());
        /* 如果笔记本为空，笔记列表视图/web视图都显示缺省笔记页面 */
        if (bRet === false) {
            isNeedShowDefaultView = true
        }

        /*
         * 触发显示NoteList笔记列表视图 和 noteWeb笔记web视图
         * 如果该笔记本为空，触发显示缺省页面，隐藏笔记列表视图和笔记web视图
         */
        noteListViewRoot.sigShowNotelistWebView(isNeedShowDefaultView, noteListViewRoot.isRecycleroot())

        //初始化的时候是否显示已置顶
        topText.visible = noteListViewRoot.getNoteListIsTop()

        /* 清楚多选功能相关 */
        rightView.multiSelectViewAlias.visible = false
        m_multiSelectArray = []
        m_iSelectedNoteSum = 1
    }

    /* 响应笔记本树层新建笔记 */
    function treeViewMenuAddNote(folderContentId) {
        m_folderContentId = folderContentId;
        noteListViewRoot.sigShowNotelistWebView(false, false)
        noteListViewRoot.addNote()
    }

    /* 调用model数据层新建笔记 */
    function addNote() {
        var bRet = false
        bRet = modellist.addNote(m_folderContentId)
        if (bRet) {
            m_bAutoExclusive = true
            m_multiSelectArray = []
            m_iSelectedNoteSum = 1
            rightView.multiSelectViewAlias.visible = false
            var iIndex = noteListViewRoot.m_abstractListAlias.listViewAlias.count - 1
            noteListViewRoot.m_abstractListAlias.listViewAlias.setListItemCheck(iIndex)
            noteListViewRoot.m_dataListAlias.listViewAlias.setListItemCheck(iIndex)
            noteListViewRoot.noteAmountChanged();

            noteListViewRoot.m_abstractListAlias.listViewAlias.clearListItemChecked()
            noteListViewRoot.m_dataListAlias.listViewAlias.clearListItemChecked()
            listNoteSort();
        }

        return bRet
    }

    /* 弹出永久删除弹窗 */
    function deleteNoteConfirm() {
        /* 加载弹窗 */
        popup.dialogType = InteractionManager.PermanentlyDeleteNotes
        popup.source = "ConfirmPopup.qml"
        popup.item.sigAccepted.connect(deleteNote)
        popup.item.show()
    }

    /* 调用model层置顶笔记 */
    function topNote() {
        var index = getcurrentindex()
        modellist.setNoteTop(index)
        listNoteSort();

        topText.visible = noteListViewRoot.getNoteListIsTop()
    }

    /* 调用model层删除笔记 */
    function deleteNote() {
        var index = getcurrentindex()
        var iRet = modellist.deleteNote(index, !isRecycleroot())
        switch (iRet) {
        case 0:
            break;

            /* 1:成功删除笔记，设置相邻笔记选中高亮 */
        case 1:
            var iIndex = 0;
            var iNoteCount = noteListViewRoot.m_abstractListAlias.listViewAlias.count
            if (index === iNoteCount) {
                iIndex = index - 1
            } else {
                iIndex = index
            }
            noteListViewRoot.m_abstractListAlias.listViewAlias.setListItemCheck(iIndex)
            noteListViewRoot.m_dataListAlias.listViewAlias.setListItemCheck(iIndex)
            break;

            /* 2:笔记列表里的笔记被删空，发信号通知显示缺省页面 */
        case 2:
            noteListViewRoot.sigShowNotelistWebView(true, noteListViewRoot.isRecycleroot())
            break;

        default: break;
        }

        if (iRet !== 0) {
            noteListViewRoot.noteAmountChanged();
        }
    }


    /* 弹出清空回收站弹窗 */
    function clearRecycleConfirm() {
        popup.dialogType = InteractionManager.ClearRecycle
        popup.source = "ConfirmPopup.qml"
        popup.item.sigAccepted.connect(clearRecycle)
        popup.item.show()
    }

    /* 调用model层清除回收站笔记 */
    function clearRecycle() {
        if (modellist.clearRecylce()) {
            DTK.sendMessage(noteListViewRoot, qsTr("Trash is empty now"), "checked")
            noteListViewRoot.sigShowNotelistWebView(true, noteListViewRoot.isRecycleroot())
            noteListViewRoot.noteAmountChanged();
        }
    }

    /* 调用model层,查看当前是否有置顶数据*/
    function getNoteListIsTop() {
        var bRet = modellist.getNoteListTop()
        return bRet
    }

    /* 调用model层恢复回收站笔记 */
    function restoreNote(noteIndexList) {
        var iRet = modellist.restoreNote(noteIndexList)
        if (iRet) {
            if (iRet === 2) {
                noteListViewRoot.sigShowNotelistWebView(true, noteListViewRoot.isRecycleroot())
            }
            DTK.sendMessage(noteListViewRoot, qsTr("The note is restored to Quick Notes"), "checked")
            noteListViewRoot.noteAmountChanged();
        }
    }

    /* 判断是否是回收站根节点 */
    function isRecycleroot() {
        var bRet = false

        var folderContentType = modellist.getFolderContentType(m_folderContentId)
        if (folderContentType === InteractionManager.RECYCLEROOT) {
            bRet = true
        }

        return bRet
    }

    /* 响应缺省页面新建笔记 */
    function onDefaultPageAddNoteSig() {
        noteListViewRoot.sigShowNotelistWebView(false, false)
        noteListViewRoot.addNote()
    }

    /*调用model层排序算法*/
    function listNoteSort() {
        var iCurrent = modellist.sortView(getcurrentindex())
        setNewIndex(iCurrent);
    }

    /*设置新的INDEX*/
    function setNewIndex(index) {
        if (m_selectFileType === InteractionManager.AbstractView) {
            noteListViewRoot.m_abstractListAlias.listViewAlias.setListItemCheck(index);
        } else if (m_selectFileType === InteractionManager.CardView) {
            noteListViewRoot.m_cardList.listViewAlias.setListItemCheck(index);
        } else if (m_selectFileType === InteractionManager.ListView) {
            noteListViewRoot.m_dataListAlias.listViewAlias.setListItemCheck(index);
        }
    }

    /* 发信号通知笔记列表里的笔记数量发生变化，笔记新建/删除 */
    function noteAmountChanged() {
        noteListViewRoot.sigNoteAmountChanged();
    }

    /* 销毁弹窗 */
    function popupDestory() {
        popup.source = ""
    }
    /* 搜索 */
    function search(key, searchType) {
        /* 结束搜索 */
        if (key === "") {
            noteListTitle.text = qsTr(m_noteBookName)
            if (searchType === InteractionManager.NOTEBOOKROOT) {
                modellist.setSearchKey(key)
                editItemSource()
            }
            else {
                modellist.loadSearchNotes(key, searchType)
            }
        } else {
            if (searchType === InteractionManager.NOTEBOOKROOT) {
                noteListTitle.text = qsTr("All Notes")
            }
            modellist.loadSearchNotes(key, searchType)
        }
    }

    function statiSelectedNotes(iIndex, bChecked) {
        if (m_iSelectedNoteSum >= 2 && rightView.multiSelectViewAlias.visible === false) {
            rightView.multiSelectViewAlias.visible = true
        } else if (m_iSelectedNoteSum === 1 && rightView.multiSelectViewAlias.visible === true) {
            rightView.multiSelectViewAlias.visible = false
        }
    }
}
