import QtQuick 2.4
import QtQuick.Layouts 1.11
import QtQuick.Window 2.11
import QtQuick.Controls 2.4
import org.deepin.dtk 1.0
import NoteBacked 1.0

DialogWindow {
    id: dialog
    width: 400
    minimumWidth: 400
    maximumWidth: 400
    icon: "deepin-voice-note"
    visible: false
    modality: Qt.ApplicationModal

    property int result: Dialog.Rejected
    /* 多选数目 */
    property int multipleNumber
    property string query
    property string remind
    property string rejectText: qsTr("Cancel")
    property string acceptText: qsTr("Confirm")

    /* 信号 */
    signal sigResult(int type,string text)


    ColumnLayout {
        width: parent.width
        Label {
            Layout.alignment: Qt.AlignHCenter
            font: DTK.fontManager.t5
            text: query
        }
        Item {
            id: name
            height: 10
        }
        ColumnLayout {
            spacing: 10
            LineEdit {
                id: lineedit
                text: remind
                focus: true
                placeholderText: qsTr("Enter a notebook name")
                maximumLength: 64
                Layout.fillWidth: true
                onAccepted: {
                    if (text.trim().length !== 0) {
                        done(Dialog.Accepted)
                    }
                }
            }

            ComboBox {
                ColorSelector.family: Palette.CommonColor
                Layout.fillWidth: true
                displayText: qsTr("")
                model: ListModel {
                    ListElement { device: "icon1"}
                    ListElement { device: "icon2"}
                }

                DciIcon {
                    id: boxicon
                    x: 12
                    anchors.verticalCenter: parent.verticalCenter
                    name: "folder"
                    width: 20
                    height: 20
//                    mode: itemDele.ColorSelector.controlState
//                    palette: DTK.makeIconPalette(itemDele.palette)
                    theme: DTK.themeType
                }

                Label {
                    id: boxText
                    anchors.left: boxicon.right
                    anchors.leftMargin: 12
                    anchors.verticalCenter: parent.verticalCenter
                    text: qsTr("Icon")
                }
            }

            ComboBox {
                ColorSelector.family: Palette.CommonColor
                Layout.fillWidth: true
                displayText: qsTr("")
                model: ListModel {
                        ListElement { device: "color1"}
                        ListElement { device: "color2" }
                }


                Rectangle {
                    id: boxrect
                    x: 12
                    anchors.verticalCenter: parent.verticalCenter
                    width: 18
                    height: 18
                    radius: 4
                    border.color: Qt.rgba(0, 0, 0, 0.1)
                    border.width: 1
                    opacity: 1
                    color: "#EC8F0B"
                }

                Label {
                    id: boxText2
                    anchors.left: boxrect.right
                    anchors.leftMargin: 12
                    anchors.verticalCenter: parent.verticalCenter
                    text: qsTr("Color")
                }
            }
        }

        RowLayout {
            Layout.alignment: Qt.AlignBottom | Qt.AlignHCenter
            Layout.bottomMargin: 10
            Layout.topMargin: 10
            Layout.fillWidth: true

            Button {
                text: rejectText
                Layout.preferredWidth: 175
                font: DTK.fontManager.t6
                onClicked: done(Dialog.Rejected)
                Keys.onReturnPressed: done(Dialog.Rejected)
                Keys.onEscapePressed: done(Dialog.Rejected)

            }

            Item {Layout.fillWidth: true}

            WarningButton {
                text: acceptText
                Layout.preferredWidth: 175
                font: DTK.fontManager.t6
                enabled: lineedit.text.trim() ? true:false
                Layout.alignment: Qt.AlignRight
                onClicked: done(Dialog.Accepted)
                Keys.onEscapePressed: done(Dialog.Rejected)
                Keys.onReturnPressed: done(Dialog.Accepted)
            }
        }
    }

    function setDialogTypeAndText(type,text) {
        switch(type) {
        case(InteractionManager.CreatNotebook):
        case(InteractionManager.CreatMulNotebook):
            query = qsTr("New Notebook")
            remind = text
            acceptText = qsTr("Create")
            rejectText = qsTr("Cancel")
            break
        case(InteractionManager.ModifyNotebook):
            query = qsTr("Edit Notebook")
            remind = text
            acceptText = qsTr("Confirm")
            rejectText = qsTr("Cancel")
            break
        }
    }

    function done(type) {
        result = type
        close()
    }

    function open(){
        visible = true
    }

    onClosing:{
        sigResult(result, lineedit.text.trim())
        close.accepted = true;
    }
}
