import QtQuick 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 1.4
import QtQml.Models 2.11
import QtQml 2.11
import org.deepin.dtk 1.0
import NoteBacked 1.0

ItemDelegate {
    id: itemDele
    width: noteBookTree.width
    height: 40
    backgroundVisible: isItemHover && !itemDele.noteBooksetStyle()
    checkable: !itemDele.noteBooksetStyle()
    checked: styleData.selected

    property bool isItemHover: false
    property bool isMouseRectInText: false

    DciIcon {
        id: nodeTextImage
        x: styleData.depth === 0 ? (itemDele.noteBooksetStyle() ? 0 : 5): expandArrow.width * styleData.depth + 5
        anchors.verticalCenter: parent.verticalCenter
        name: itemDele.notebookIcon()
        width: itemDele.noteBooksetStyle() ? 0 : 16
        height: itemDele.noteBooksetStyle() ? 0 : 16
        mode: itemDele.ColorSelector.controlState
        palette: DTK.makeIconPalette(itemDele.palette)
        theme: DTK.themeType
    }

    DciIcon {
        id: expandArrow
        x: 0
        visible: styleData.hasChildren
        name: styleData.isExpanded ? "arrow_ordinary_down" : "arrow_ordinary_right"
        anchors.verticalCenter: parent.verticalCenter
        sourceSize.width: 10
        sourceSize.height: 10
        anchors.right: itemDele.noteBooksetStyle() ? itemDele.right : nodeTextImage.left
        anchors.rightMargin: itemDele.noteBooksetStyle() ? 10 : 0
        mode: itemDele.ColorSelector.controlState
        palette: DTK.makeIconPalette(itemDele.palette)
        theme: DTK.themeType
    }

    DciIcon {
        id: iconButton
        visible: (!itemDele.noteBooksetStyle() && itemDele.isItemHover) ? true : false
        name: "more"
        width: itemDele.noteBooksetStyle() ? 0 : 16
        height: itemDele.noteBooksetStyle() ? 0 : 16
        anchors {
            right: itemDele.right
            rightMargin: 10
            verticalCenter: parent.verticalCenter
        }
        mode: itemDele.ColorSelector.controlState
        palette: DTK.makeIconPalette(itemDele.palette)
        theme: DTK.themeType
    }

    Label {
        id: nameText
        visible: !editRename.visible
        anchors.left: itemDele.noteBooksetStyle() ? itemDele.left : nodeTextImage.right
        anchors.leftMargin: itemDele.noteBooksetStyle() ? 0 : 3
        anchors.right: itemDele.noteBooksetStyle() ? expandArrow.left : iconButton.left
        anchors.rightMargin: itemDele.noteBooksetStyle() ? 0 : 5
        anchors.verticalCenter: parent.verticalCenter
        color: styleData.selected ? (DTK.themeType === 1 ? "#FFFFFF" : "#000000") : (DTK.themeType === 1 ? "#000000" : "#FFFFFF")
        opacity: styleData.selected ? 1 : (itemDele.noteBooksetStyle() ? 0.3 : 0.7)
        elide: styleData.elideMode
        text: modeltree.slotGetItemName(styleData.index)
        font: itemDele.noteBooksetStyle() ? DTK.fontManager.t7 : DTK.fontManager.t6
//        Drag.active:itemTextMouse.drag.active
        Drag.dragType: Drag.Automatic;      //选择自动开始拖动
        Drag.supportedActions: Qt.CopyAction;   //选择复制数据到DropArea
        Drag.onDragFinished: {              //拖拽结束
        }

        ToolTip {
            id: tooltip
            x: itemMouse.mouseX - 30 - styleData.depth*10
            y: itemDele.height - 8
            delay: 1000
            text: modeltree.slotGetItemName(styleData.index)
            font: DTK.fontManager.t7
            visible: itemDele.isMouseRectInText && styleData.depth && itemMouse.containsMouse && !root.isRenamed
        }
    }

    Label {
        id: countText
        visible: itemDele.textVisiableState()
        anchors.right: itemDele.right
        anchors.rightMargin: itemDele.noteBooksetStyle() ? 0 : 10
        anchors.verticalCenter: parent.verticalCenter
        color: styleData.selected ? (DTK.themeType === 1 ? "#FFFFFF" : "#000000") : (DTK.themeType === 1 ? "#000000" : "#FFFFFF")
        opacity: styleData.selected ? 0.9 : 0.6
        elide: styleData.elideMode
        text: modeltree.slotGetItemCount(styleData.index)
        font: DTK.fontManager.t8
//        Drag.active:itemTextMouse.drag.active
        Drag.dragType: Drag.Automatic;      //选择自动开始拖动
        Drag.supportedActions: Qt.CopyAction;   //选择复制数据到DropArea
        Drag.onDragFinished: {              //拖拽结束
        }
    }

    MouseArea {
        id: itemMouse
        anchors.fill: parent
        hoverEnabled: true
        acceptedButtons: Qt.RightButton | Qt.LeftButton
        onClicked: {
            rightmenu.index = styleData.index
            sel.isNoteBookroot = itemDele.noteBooksetStyle()
            sel.depth = styleData.depth
            sel.selrow = styleData.row
            if(!itemDele.noteBooksetStyle()) {
                sel.setCurrentIndex(styleData.index, 0x0010)
            }

            //右键菜单功能
            if(mouse.button & Qt.RightButton || itemDele.onHoverIconPostion(mouseX,mouseY)) {
                itemDele.noteBookRightMenu(mouseX,mouseY)
            }
            //点击节点图标
            itemDele.onClickNodePostion(mouseX,mouseY)
            root.forceActiveFocus()
        }

        onDoubleClicked: {
            //在文本所在区域双击才触发重命名
            if(!itemDele.noteBooksetStyle()) {
                sel.setCurrentIndex(styleData.index, 0x0010)
            }
            if(onHoverTextPostion(mouseX,mouseY)) {
                itemDele.renameNodebook(styleData.index)
            }
        }

        onEntered: {
            if(!styleData.selected)
                itemDele.isItemHover = true
        }
        onExited: {
            itemDele.isItemHover = false
        }

        onPositionChanged: {
            root.scrollvbar.active = false
            if(styleData.selected) {
                onHoverIconPostion(mouseX,mouseY)
            }
            onHoverTextPostion(mouseX,mouseY)
        }
    }

    //输入框背景
    Rectangle {
        id: editItem
        width: editRename.width + 4
        height: editRename.height + 2
        anchors.left: nodeTextImage.right
        anchors.leftMargin: 1
        anchors.verticalCenter: parent.verticalCenter
        visible: styleData.selected && root.isRenamed
        radius: 8
    }

    LineEdit {
        id: editRename
        text: styleData.selected ? modeltree.slotGetItemName(styleData.index) : ""
        focus: true
        anchors.left: nodeTextImage.right
        anchors.leftMargin: 3
        anchors.verticalCenter: parent.verticalCenter
        visible: styleData.selected && root.isRenamed
        font: DTK.fontManager.t7
        width: (noteBookTree.width - nodeTextImage.width - expandArrow.width * styleData.depth - 13)
        height: 30
        maximumLength: 64
        onEditingFinished: {
            if(root.isRenamed) {
                root.isRenamed = false
                editRename.text = editRename.text.trim()
                if(editRename.text !== "" && editRename.text !== nameText.text) {
                    modeltree.slotRenameNoteBooks(styleData.index,editRename.text)
                    root.sigShowNoteList(modeltree.slotGetItemFolderId(styleData.index), editRename.text);
                }
            }
        }
    }

    //判断是否当前为笔记本根节点
    function noteBooksetStyle() {
        if(styleData.depth === 0 && styleData.row === 2) {
            root.rootindex = styleData.index
            return true
        }
        return false
    }

    function notebookIcon() {
        if(itemDele.noteBooksetStyle()) {
            return ""
        } else {
            if(modeltree.slotGetItemIcon(styleData.index) === "") {
                return "notebook"
            } else {
                return modeltree.slotGetItemIcon(styleData.index)
            }
        }
    }

    //右键菜单弹出
    function noteBookRightMenu(x,y) {
        rightmenu.posx = mapToItem(noteBookTree,x,y).x
        rightmenu.posy = mapToItem(noteBookTree,x,y).y
        if(styleData.row===0 && styleData.depth === 0) {
            rightmenu.treequickNoteMenu.open();
        } else if (styleData.row===1 && styleData.depth === 0) {
            rightmenu.treerecycleMenu.open();
        } else  if(styleData.row ===2 && styleData.depth === 0){
            //点击笔记本项不显示右键菜单
        } else  {
            rightmenu.treeRightMenu.open();
        }
    }

    //判断点击位置是否为节点图标位置
    function onClickNodePostion(x,y) {
        if(expandArrow.x <= x && x <= expandArrow.x + 10 && expandArrow.y <= y && y <= expandArrow.y + 10) {
            if (!styleData.isExpanded) {
                noteBookTree.expand(styleData.index)
            }
            else {
                noteBookTree.collapse(styleData.index)
            }
        }
    }

    //判断鼠标hover位置是否为更多图标位置
    function onHoverIconPostion(x,y) {
        if(iconButton.x <= x && x <= iconButton.x + 16 && iconButton.y <= y && y <= iconButton.y + 16) {
            itemDele.isItemHover = true
        }
        else {
            itemDele.isItemHover = false
        }
        return itemDele.isItemHover
    }

    //判断鼠标是否在文本区域
    function onHoverTextPostion(x,y) {
        if(nameText.x <= x && x <= nameText.x + nameText.width) {
            itemDele.isMouseRectInText = true
        }
        else {
            itemDele.isMouseRectInText = false
        }
        return itemDele.isMouseRectInText
    }

    function textVisiableState() {
        if(styleData.selected) {
            if(!root.isRenamed && !itemDele.isItemHover && modeltree.slotGetItemCount(styleData.index)!== "")
                return true
            return false

        } else {
            if(itemDele.noteBooksetStyle() || modeltree.slotGetItemCount(styleData.index)=== "")
                return false
            return !itemDele.isItemHover
        }
    }

    //重命名开启
    function renameNodebook(id) {
        if(id === styleData.index) {
            editRename.text = modeltree.slotGetItemName(styleData.index)
            editRename.forceActiveFocus(Qt.MouseFocusReason)
            editRename.selectAll()
            root.isRenamed = styleData.depth
        }
    }

    //重命名完成
    function renameNodebookfinish(index) {
        if(root.isRenamed && index === styleData.index) {
            root.isRenamed = false
            editRename.text = editRename.text.trim()
            if(editRename.text !== "" && editRename.text !== nameText.text) {
                modeltree.slotRenameNoteBooks(styleData.index,editRename.text)
                root.sigShowNoteList(modeltree.slotGetItemFolderId(styleData.index), editRename.text);
            }
        }
    }

    Component.onCompleted: {
        //右键菜单重命名消息绑定
        rightmenu.signalRename.connect(itemDele.renameNodebook);
        root.signalRenamefinish.connect(itemDele.renameNodebookfinish);

        if(!styleData.depth && !styleData.row) {
            sel.setCurrentIndex(styleData.index,0x0010);
        }

        if(noteBooksetStyle()) {
            noteBookTree.expand(styleData.index);
        }
    }
}
