import QtQuick 2.0
import QtQuick.Window 2.11
import QtQuick.Layouts 1.0
import QtQuick.Controls 2.4 as C
import QtQuick.Controls 1.4
import QtQml.Models 2.11
import QtQml 2.11
import org.deepin.dtk 1.0
import NoteBacked 1.0
Rectangle {
    id: root
    width: leftView.width
    height: leftView.height
    color: leftView.color
    anchors.top: parent.top
    anchors.margins: 10
    signal  sigShowNoteList(int folderContentId, string noteBookName)
    signal  signalAddNote(int folderContentId)
    signal  signalDeleteAll()
    signal  signalNoticeMenuStyle(int type)
    signal  signalRenamefinish(var index)
    property bool isRenamed: false
    property alias scrollvbar: vbar
    property var rootindex

    //用于节点选中
    ItemSelectionModel {
        id: sel
        model: modeltree
        property bool isNoteBookroot: false
        property int depth: 0
        property int selrow: 0
        onCurrentChanged: {
            //左侧列表跟随当前项变化选择切换笔记列表信号
            if(!isNoteBookroot) {
                rightmenu.index = current
                root.sigShowNoteList(modeltree.slotGetItemFolderId(current), modeltree.slotGetItemName(current));
                signalNoticeMenuStyle(root.getFolderType())
                signalRenamefinish(previous)
            }
        }
    }

    LeftTreeMode {
        id: modeltree
    }

    Loader {
        id: popup
        property int dialogType
        property string name
        onLoaded: {
            if(dialogType < InteractionManager.CreatNotebook) {
                item.setDialogType(dialogType)
                item.sigDestroy.connect(root.popupDestory)
            } else {
                item.setDialogTypeAndText(dialogType,name)
            }
        }
    }

    MouseArea {
        id: rootItemMouse
        anchors.fill: parent
        acceptedButtons: Qt.RightButton | Qt.LeftButton
        propagateComposedEvents: true
        onClicked: {
            signalRenamefinish(rightmenu.index)
            mouse.accepted = false
        }
    }

    Rectangle {
        id: rect
        width: leftView.width
        color: leftView.color
        height: root.treeheight()
        TreeView {
            id: noteBookTree
            width: root.width
            height: rect.height
//            anchors.top: parent.top
            anchors.margins: 10
//            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            headerVisible: false
            alternatingRowColors: false
            backgroundVisible : false   //隐藏背景
            style: LeftTreeStyle {}
            model: modeltree
            selection: sel
            frameVisible: false
            clip: true
            TableViewColumn {
                width: noteBookTree.width
                title: "Name"
                role: "display"
            }
            verticalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
            LeftRightMenu {
                id: rightmenu
            }
        }

        MouseArea {
            id: rectMouse
            anchors.fill: parent
            acceptedButtons: Qt.MiddleButton
            onWheel: {
                var length = noteBookTree.flickableItem.contentHeight - noteBookTree.flickableItem.height
                if(length > 0) {
                    vbar.active = true
                    if(wheel.angleDelta.y > 0 ) {
                        if(noteBookTree.flickableItem.contentY > 0)
                            noteBookTree.flickableItem.contentY = noteBookTree.flickableItem.contentY - 10
                        else
                            noteBookTree.flickableItem.contentY = 0
                    }
                    else {
                        if(noteBookTree.flickableItem.contentY < length)
                            noteBookTree.flickableItem.contentY = noteBookTree.flickableItem.contentY + 10
                        else
                            noteBookTree.flickableItem.contentY = length
                    }
                }
            }
        }

        ScrollBar {
            id: vbar
            orientation: Qt.Vertical
            hoverEnabled: true
            size: noteBookTree.height / noteBookTree.flickableItem.contentHeight
            anchors.top: parent.top
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            onPositionChanged: {
                noteBookTree.flickableItem.contentY = position * noteBookTree.flickableItem.contentHeight
            }
        }

        Connections {
            target: noteBookTree.flickableItem
            onContentYChanged: {
                vbar.position = noteBookTree.flickableItem.contentY / noteBookTree.flickableItem.contentHeight
            }
        }
    }

    Component.onCompleted: {
        modeltree.signalCreateNoticeToQml.connect(root.createNoticesetSelect);
        modeltree.signalDeleteNoticeToQml.connect(root.deleteNoticesetSelect)
    }

    Button {
        id:creatButton
        height: 36
        width:root.width - 2*root.anchors.margins
        y: root.height - creatButton.height - 3*root.anchors.margins
        anchors.margins: root.buttonmargins()
        anchors.top: rect.bottom
        anchors.horizontalCenter: root.horizontalCenter
        Text{
            text: qsTr("New Notebook")
            anchors.horizontalCenter: creatButton.horizontalCenter
            anchors.verticalCenter: creatButton.verticalCenter
            font: DTK.fontManager.t6
        }
        onClicked: {
            creatNoteBookDialog(InteractionManager.CreatNotebook,"")
        }
    }

    //通知数据更新的信号
    function noticeUpdateTreeData() {
        modeltree.slotupdateData()
    }

    //返回笔记树的显示区域高度
    function treeheight() {
        if(noteBookTree.flickableItem.contentHeight >= (root.height - creatButton.height - 3*root.anchors.margins)) {
            return (root.height - creatButton.height - 3*root.anchors.margins)
        }
        else {
            return noteBookTree.flickableItem.contentHeight
        }
    }

    //返回按钮和tree 之间的间距距离
    function buttonmargins() {
        //顶部rect间距10,按钮和tree 间距10 ,底部和按钮间距10
        if(noteBookTree.flickableItem.contentHeight >= (root.height - creatButton.height - 3*root.anchors.margins))
            return root.anchors.margins
        else
            return (root.height - 2*root.anchors.margins - creatButton.height - noteBookTree.flickableItem.contentHeight)
    }

     /* 删除笔记本 */
    function deleteNoticesetSelect(index) {
        sel.selrow = 2
        sel.depth = 0
        sel.setCurrentIndex(index,0x0010)
    }

    /* 新建笔记本 */
    function createNoticesetSelect(index) {
        sel.selrow = 2
        sel.setCurrentIndex(index,0x0010)
    }

    /* 弹出删除笔记本弹窗 */
    function deleteNoteBookDialog() {
        if (sel.depth <= 0) {
            return
        }
        popup.dialogType = InteractionManager.DeleteNotebook
        popup.source = "../notelistview/ConfirmPopup.qml"
        popup.item.sigAccepted.connect(deleteNotesInfo)
        popup.item.show()
    }

    /* 调用model层删除笔记本和笔记 */
    function deleteNotesInfo() {
        console.info(rightmenu.index)
        modeltree.slotDeleteNoteBookFolder(rightmenu.index)
    }

    /* 销毁弹窗 */
    function popupDestory() {
        popup.source = ""
    }

    /* 弹出创建或者修改笔记本弹窗 */
    function creatNoteBookDialog(type,text) {
        if (sel.depth <= 0 && type !== InteractionManager.CreatNotebook) {
            return
        }
        popup.dialogType = type
        popup.name =text
        popup.source = "CreatNotebookPopup.qml"
        popup.item.sigResult.connect(creatNotesInfo)
        popup.item.open()
    }

    /* 调用model层创建或者修改笔记本名称和图标颜色等 */
    function creatNotesInfo(result,text) {
        popup.source = ""
        if (result === C.Dialog.Accepted) {
            if(popup.dialogType === InteractionManager.CreatNotebook) {
                sel.clearCurrentIndex()
                modeltree.slotaddNewNoteBook(text)
                if(!noteBookTree.isExpanded(rootindex))
                    noteBookTree.expand(root.rootindex);
            }
            else if(popup.dialogType === InteractionManager.CreatMulNotebook) {
                sel.clearCurrentIndex()
                modeltree.slotAddChilderNode(rightmenu.index,text)
                if(!noteBookTree.isExpanded(rightmenu.index))
                    noteBookTree.expand(rightmenu.index)
            }
            else if(popup.dialogType === InteractionManager.ModifyNotebook) {
                modeltree.slotRenameNoteBooks(rightmenu.index,text)
                sigShowNoteList(modeltree.slotGetItemFolderId(rightmenu.index), text);
            }
        }
    }

    /* 获取当前选中的笔记本类型 */
    function getFolderType() {
        if (!sel.depth && !sel.selrow) {
            return InteractionManager.QUICKNOTEROOT
        } else if (!sel.depth && sel.selrow ===1) {
            return InteractionManager.RECYCLEROOT
        }else {
           return InteractionManager.NOTEBOOKNORMAL
        }
    }

    /*快捷键调用重命名*/
    function shortCutRename() {
         rightmenu.signalRename(rightmenu.index)
    }
}


