import QtQuick 2.15
import QtWebChannel 1.15
import QtWebEngine 1.10
import NoteBacked 1.0
import QtQuick.Controls 1.4
import QtQuick.Layouts 1.11
import "../audioview"

Item {
    property alias jsBackend: channelContent
    property alias editor: richTextEditor
    property alias audioManager: audioManager
    property int currentNoteId: 0
    property string searchKey: ""
    property int currentMenuType: 0
    property string currentMenuParam: ""
    property bool webLoadFinsh: false
    property bool editorIsReadonly: false
    signal insertVoiceFinsh();

    JSContent {
        id: channelContent
        noteId: currentNoteId
        WebChannel.id: "jsBackend"
        signal sigCallJsTriggerDelete();
        signal sigCallJsToggleToolbar(bool flag);
        signal sigCallJsAddTable(int rows, int columns);
        signal sigTabCallJsOperation(string action);
        signal sigCallJsToolbarHide();
        signal sigCallJsInsertVoice(string data);
        signal sigCallJsSetPlayState(var flag);
        function slotSetMenutype(menuType, menuParam){
            currentMenuType = menuType
            currentMenuParam = menuParam
        }
        function slotSetWebLoadFinsh(){
            webLoadFinsh = true;
        }
        function slotShowToolbar(flag){
           sigCallJsToggleToolbar(flag)
        }
        function slotPlayAudio(data, flag){
            audioManager.startPlay(data, flag)
        }

        onNoteIdChanged: {
            richTextEditor.findText(searchKey)
        }
    }

    WebChannel {
        id: channel
        registeredObjects: [channelContent]
    }

    RichTextEditorMenu {
        id: menu
    }
    ColumnLayout{
        anchors.fill: parent
        WebEngineView {
            Layout.fillWidth: true
            Layout.fillHeight: true
            visible: (jsBackend.noteId > 0 && webLoadFinsh)
            id: richTextEditor
            url: webPath + "dist/index.html"
            webChannel: channel
            onContextMenuRequested: function(request) {
                request.accepted = true;
                switch (currentMenuType){
                case 0:
                    menu.showTextMenu(request, editorIsReadonly)
                    break;
                case 1:
                    menu.showPictureMenu(currentMenuParam, editorIsReadonly)
                    break;
                case 2:
                    menu.showVoiceMenu(currentMenuParam, editorIsReadonly)
                    break
                case 3:
                    menu.showTableMenu(currentMenuParam, editorIsReadonly)
                    break;
                default:
                    break;
                }
            }
            DropArea {
                anchors.fill: editor
                enabled: true
                onDropped:{
                    if(drop.hasUrls){
                        jsBackend.slotInsertImages(drop.urls)
                    }
                }
            }

            Component.onCompleted: {
                jsBackend.sigPasteDefault.connect(function() {editor.triggerWebAction(WebEngineView.Paste)});
                jsBackend.sigCallJsSetReadonly.connect(function(ret){editorIsReadonly = ret});
            }
        }
        AudioManager{
            id: audioManager
            visible: false
            Layout.fillWidth: true
            Layout.preferredHeight: 70
            Layout.minimumHeight: 70
            Layout.maximumHeight: 70
            Layout.bottomMargin: 10
            Layout.leftMargin: 10
            Layout.rightMargin: 10
            Layout.alignment: Qt.AlignBottom
            Component.onCompleted: {
                audioManager.audioRecorder.audioControl.sigRecordFinsh.connect(function(data){
                    insertVoice(data)
                });
            }
            audioPlayer.onPauseChanged: {
                jsBackend.sigCallJsSetPlayState(audioPlayer.pause)
            }
        }
    }

    function insertVoice(data){
        audioManager.audioRecorder.pause = false;
        var jsStr = "callJsFunction('insertVoice'," +  "[" + data + "])";
        editor.runJavaScript(jsStr, function(){
            insertVoiceFinsh()
        });
    }

    function stopAudio(){
        if(audioManager.visible){
            audioManager.stopAudioOper();
        }
    }
}
