import QtQuick 2.0
import Qt.labs.platform 1.0 as LAB
import QtQuick.Controls 1.4
import QtWebEngine 1.10
import NoteBacked 1.0
import org.deepin.dtk 1.0 as D
import QtQuick.Layouts 1.11

Item {
    D.DialogWindow{
        id: dialog
        width: 400
        visible: false
        icon: "dialog-warning"
        modality: Qt.ApplicationModal
        ColumnLayout {
            width: parent.width
            spacing: 22
            D.Label {
                id: dialogMessage
                Layout.alignment: Qt.AlignHCenter
                font: D.DTK.fontManager.t5
            }

            D.Button {
                text: qsTr("OK")
                focus: true
                visible: true
                Layout.preferredWidth: parent.width
                Layout.bottomMargin: 10
                font: D.DTK.fontManager.t6
                onClicked: dialog.close()
                Keys.onReturnPressed: dialog.close()
                Keys.onEscapePressed: dialog.close()

            }

        }
    }

    Menu {
        id: textMenu
        property bool  showAiAction: true
        property bool  isReading: false
        property bool canSelectAll: true
        property bool canCopy: false
        property bool canCut: false
        property bool canDelete: false
        property bool canPaste: false
        property bool canTextToSpeech: false
        property bool canSpeechTextTo: false
        property bool canTrans: false
        property bool isReadonly: false
        MenuItem{
          text: qsTr("Copy")
          enabled: textMenu.canCopy
          onTriggered:{
              editor.triggerWebAction(WebEngineView.Copy)
          }
        }
        MenuItem{
          text: qsTr("Cut")
          enabled: textMenu.canCut
          visible: !textMenu.isReadonly
          onTriggered:{
              editor.triggerWebAction(WebEngineView.Cut)
          }
        }
        MenuItem{
          text: qsTr("Paste")
          enabled: textMenu.canPaste
          visible: !textMenu.isReadonly
          onTriggered:{
              jsBackend.slotPaste()
          }
        }
        MenuItem{
          text: qsTr("Select All")
          enabled: textMenu.canSelectAll
          onTriggered:{
              jsBackend.sigCallJsToolbarHide();
              editor.triggerWebAction(WebEngineView.SelectAll)
          }
        }
        MenuSeparator {
            visible: !textMenu.isReadonly
        }
        MenuItem{
            text: qsTr("Delete")
            enabled: textMenu.canDelete
            visible: !textMenu.isReadonly
            onTriggered:{
                jsBackend.sigCallJsTriggerDelete()
            }
        }
        MenuSeparator {
            visible: textMenu.showAiAction
        }
        MenuItem{
            text: qsTr("Text to Speech")
            visible: !textMenu.isReadonly && textMenu.showAiAction
            enabled: textMenu.canTextToSpeech

        }
        MenuItem{
            text: textMenu.isReading ? qsTr("Stop reading") : qsTr("Speech to Text")
            visible: textMenu.showAiAction
        }
        MenuItem{
            text: qsTr("Translate")
            visible: textMenu.showAiAction
        }
    }
    Menu {
        id: pictureMenu
        property string srcUrl: ""
        property bool isReadonly: false
        MenuItem{
            text: qsTr("View")
            onTriggered:{
                jsBackend.slotOpenFile(pictureMenu.srcUrl)
            }
        }
        MenuSeparator { }
        MenuItem{
            text: qsTr("Copy")
            onTriggered:{
                editor.triggerWebAction(WebEngineView.Copy)
            }
        }
        MenuItem{
            text: qsTr("Cut")
            onTriggered:{
                editor.triggerWebAction(WebEngineView.Cut)
            }
            visible: !pictureMenu.isReadonly
        }
        MenuItem{
            text: qsTr("Paste")
            onTriggered:{
                jsBackend.slotPaste()
            }
            visible: !pictureMenu.isReadonly
        }
        MenuItem{
            text: qsTr("Select All")
            onTriggered:{
                jsBackend.sigCallJsToolbarHide();
                editor.triggerWebAction(WebEngineView.SelectAll)
            }
        }
        MenuSeparator {visible: !pictureMenu.isReadonly}
        MenuItem{
            text: qsTr("Delete")
            onTriggered:{
                jsBackend.sigCallJsTriggerDelete()
            }
            visible: !pictureMenu.isReadonly
        }
        MenuSeparator { }
        MenuItem{
            text: qsTr("Save as")
            onTriggered:{
                saveAsFile(pictureMenu.srcUrl, "image")
            }
        }
    }
    Menu {
        id: tableMenu
        MenuItem{
            text: qsTr("Insert rows above")
            onTriggered:{
                jsBackend.sigTabCallJsOperation("mceTableInsertRowBefore")
            }
        }
        MenuItem{
            text: qsTr("Insert rows below")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableInsertRowAfter")
            }
        }
        MenuItem{
            text: qsTr("Insert columns to the left")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableInsertColBefore")
            }
        }
        MenuItem{
            text: qsTr("Insert columns to the right")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableInsertColAfter")
            }
        }
        MenuSeparator { }
        MenuItem{
            text: qsTr("Delete row")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableDeleteRow")
            }
        }
        MenuItem{
            text: qsTr("Delete column")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableDeleteCol")
            }
        }
        MenuItem{
            text: qsTr("Delete table")
            onTriggered:{
                sBackend.sigTabCallJsOperation("mceTableDelete")
            }
        }
    }

    LAB.FileDialog {
        id: fileDialogSaveAs
        property string srcUrl: ""
        title: "Please choose a file"
        folder: LAB.StandardPaths.writableLocation(LAB.StandardPaths.PicturesLocation)
        fileMode: LAB.FileDialog.SaveFile
        onAccepted: {
            var ret = jsBackend.slotCopyFile(srcUrl, fileDialogSaveAs.file)
            if(ret > 0){
                dialogMessage.text = (ret === 1) ? qsTr("You do not have permission to save files there") : qsTr("Save failed")
                dialog.visible = true;
            }
        }
        onRejected: {
            console.info("Canceled")
        }
    }
    function showTextMenu(request, isReadonly){
        var editFlags = request.editFlags
        textMenu.isReadonly = isReadonly
        textMenu.canCopy = editFlags & ContextMenuRequest.CanCopy
        if(!isReadonly){
            textMenu.canCut = editFlags & ContextMenuRequest.CanCut
            textMenu.canPaste = editFlags & ContextMenuRequest.CanPaste
            textMenu.canDelete = editFlags & ContextMenuRequest.CanDelete
        }
        textMenu.canSelectAll = editFlags & ContextMenuRequest.CanSelectAll
        textMenu.showAiAction = stateManager.getState(StatesManager.StateAISrvAvailable)
        textMenu.popup()
    }
    function saveAsFile(srcUrl, defaultName){
        if(srcUrl.length){
            var name = srcUrl.slice(srcUrl.lastIndexOf("/"));
            var suffix = name.slice(name.lastIndexOf("."));
            var baseName = defaultName.length ? "/" + defaultName + suffix : name
            if(baseName.length && suffix.length){
                fileDialogSaveAs.srcUrl = srcUrl
                fileDialogSaveAs.currentFile = fileDialogSaveAs.folder + baseName
                fileDialogSaveAs.defaultSuffix = suffix
                fileDialogSaveAs.nameFilters = [suffix]
                fileDialogSaveAs.open()
            }
        }
    }

    function showVoiceMenu(request,isReadonly){
        console.info("show voice menu")
    }
    function showPictureMenu(request, isReadonly){
        pictureMenu.srcUrl = request
        pictureMenu.isReadonly = isReadonly
        pictureMenu.popup()
    }
    function showTableMenu(request, isReadonly){
        if(!isReadonly){
            tableMenu.popup()
        }
    }
}
