import QtQuick 2.11
import NoteBacked 1.0
import org.deepin.dtk 1.0

Item {
    SearchEdit {
        id: searchEdit
        implicitWidth: parent.width
        implicitHeight: parent.height
        placeholder: qsTr("Search")

        onActiveFocusChanged:{
            /* 第一次聚焦的时候加载组件 */
            if (activeFocus && loader.status === Loader.Null) {
                loader.sourceComponent = menu
            }

            if (activeFocus && text.length !== 0 && loader.status === Loader.Ready) {
                loader.item.open()
                forceActiveFocus(Qt.MouseFocusReason)
            }
        }

        onTextChanged: {
            if (text.length !==0) {
                loader.item.open()
                forceActiveFocus(Qt.MouseFocusReason)
            } else {
                for (var i = 0; i < loader.item.contentData.length; ++i) {
                    if (loader.item.contentData[i].checked) {
                        titleBar.searchSignal("", Number(loader.item.contentData[i].objectName))
                    }
                }
                stateManager.setState(StatesManager.StateSearching, false)
            }
        }

        onAccepted: {
            if (text.length === 0) {
                return
            }
            loader.item.close()
            for (var i = 0; i < loader.item.contentData.length; ++i) {
                if (loader.item.contentData[i].checked) {
                    loader.item.search(Number(loader.item.contentData[i].objectName))
                }
            }
        }

        Component.onCompleted: {
            titleBar.clearSearchEdit.connect(function() {
                clear()

                if (loader.status === Loader.Ready) {
                    loader.item.setCurrentItem()
                }

                titleBar.forceActiveFocus(Qt.OtherFocusReason)
            })

            titleBar.startSearchSignal.connect(function() {
                forceActiveFocus(Qt.MouseFocusReason)
            })
        }
    }

    Loader {
        id: loader
    }

    Component {
        id: menu
        Menu {
            id: searchMenu
            x: searchEdit.x
            y: titleBar.height - 10
            implicitWidth: searchEdit.width

            header:Label{
                text: qsTr("搜索范围")
                font: DTK.fontManager.t8
                anchors.leftMargin: 10
            }

            MenuItem {
                id: allNote
                checked: true
                checkable:true
                text: qsTr("全部笔记")
                autoExclusive: true
                height: visible ? implicitHeight : 0
                objectName: InteractionManager.NOTEBOOKROOT.toString()
                visible: folderType === InteractionManager.NOTEBOOKNORMAL
                         || folderType === InteractionManager.QUICKNOTEROOT
                onClicked:{
                    search(InteractionManager.NOTEBOOKROOT)
                }
            }

            MenuItem {
                id: currentNote
                checkable:true
                text: qsTr("当前笔记本")
                autoExclusive: true
                height: visible ? implicitHeight : 0
                objectName: InteractionManager.NOTEBOOKNORMAL.toString()
                visible: titleBar.folderType === InteractionManager.NOTEBOOKNORMAL
                onClicked:{
                    search(InteractionManager.NOTEBOOKNORMAL)
                }
            }

            MenuItem {
                id: quickNote
                checkable:true
                text: qsTr("快速笔记")
                autoExclusive: true
                height: visible ? implicitHeight : 0
                visible: titleBar.folderType === InteractionManager.QUICKNOTEROOT
                objectName: InteractionManager.QUICKNOTEROOT.toString()
                onClicked:{
                    search(InteractionManager.QUICKNOTEROOT)
                }
            }

            MenuItem {
                id: recycle
                checkable:true
                text: qsTr("回收站")
                autoExclusive: true
                height: visible ? implicitHeight : 0
                objectName: InteractionManager.RECYCLEROOT.toString()
                visible: titleBar.folderType === InteractionManager.RECYCLEROOT
                checked: visible
                onClicked:{
                    search(InteractionManager.RECYCLEROOT)
                }
            }

            onActiveFocusChanged:{
                searchEdit.forceActiveFocus(Qt.MouseFocusReason)
            }

            /* 发送搜索信号 */
            function searchConfirm(type) {
                titleBar.searchSignal(searchEdit.text, type)
            }

            /* 控制选中状态 */
            function setCurrentItem() {
                if (titleBar.folderType === InteractionManager.RECYCLEROOT) {
                    recycle.checked = true
                } else {
                    allNote.checked = true
                }
            }

            /* 搜索 */
            function search(type) {
                stateManager.setState(StatesManager.StateSearching, true)
                searchMenu.searchConfirm(type)
            }
        }
    }
}


