import QtQuick 2.11
import Qt.labs.platform 1.0
import QtQuick.Window 2.11
import QtQuick.Layouts 1.11
import QtQuick.Controls 2.3
import QtQuick.Dialogs 1.0
import NoteBacked 1.0
import org.deepin.dtk 1.0

//list视图切换弹窗
Popup {
    id: listViewSwitchPopup

    property int noteSortType
    property int recycleSortType
    property alias popupModel: popupListView.model

    x: - (width / 2) + 20
    y: titleBar.height - 8
    width: 200
    height: 325
    modal: true
    ColumnLayout {
        width: parent.width
        Label {
            text: qsTr(buttonBox.viewText)
            Layout.alignment: Qt.AlignHCenter
            font: DTK.fontManager.t5
        }

        ButtonBox {
            id: buttonBox
            Layout.alignment: Qt.AlignCenter
            property var viewTexts: {"AbstractView": "Thumbnail view", "CardView": "Card view", "ListView" : "List view"}
            property string viewText: viewTexts["AbstractView"]
            ToolButton {
                checked: true
                checkable: true;
                icon.name: "thumbnail"
                ToolTip.visible: hovered
                ToolTip.text: qsTr(buttonBox.viewTexts["AbstractView"])
                onClicked: {
                    buttonBox.viewText = buttonBox.viewTexts["AbstractView"]
                    titleBar.listViewSwitchSignal(InteractionManager.AbstractView)
                }
            }
            ToolButton {
                checkable: true;
                icon.name: "card"
                ToolTip.visible: hovered
                ToolTip.text: qsTr(buttonBox.viewTexts["CardView"])
                onClicked: {
                    buttonBox.viewText = buttonBox.viewTexts["CardView"]
                    titleBar.listViewSwitchSignal(InteractionManager.CardView)
                }
            }
            ToolButton {
                checkable: true;
                icon.name: "list"
                ToolTip.visible: hovered
                ToolTip.text: qsTr(buttonBox.viewTexts["ListView"])
                onClicked: {
                    buttonBox.viewText = buttonBox.viewTexts["ListView"]
                    titleBar.listViewSwitchSignal(InteractionManager.ListView)
                }
            }
        }

        MenuSeparator { text: qsTr("Sort by"); Layout.alignment: Qt.AlignLeft }

        ListView {
            id: popupListView

            property int defaultIndex: 0
            property int previousIndex: 0

            Layout.preferredHeight: contentHeight
            layoutDirection: Qt.RightToLeft
            spacing: 10
            model: ""

            ButtonGroup { id: clickExclusiveGroup }

            delegate: ItemDelegate{
                id:  popupDelegate
                width: 180
                text: " "
                indicatorVisible: true
                LayoutMirroring.enabled: true
                checked: model.type === (titleBar.folderType === InteractionManager.RECYCLEROOT ? recycleSortType : noteSortType)
                Label {
                    text: qsTr(model.text)
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.leftMargin: 35
                }

                icon.name: {
                    if (model.type === InteractionManager.DeteleTime
                            || model.type === InteractionManager.ModifyTime
                            || model.type === InteractionManager.CreationTime)
                        popupDelegate.checked ? (model.order === Qt.DescendingOrder ? "descending" : "ascending") : ""
                    else if (model.type === InteractionManager.NoteName)
                        popupDelegate.checked ? (model.order === Qt.DescendingOrder ? "ascending" : "descending") : ""
                    else
                        ""
                }
                icon.width: 16
                icon.height: 16

                ButtonGroup.group: clickExclusiveGroup

                onClicked: {
                    popupListView.currentIndex = index

                    popupListView.model.setProperty(popupListView.currentIndex, "order"
                                                    , model.order === Qt.DescendingOrder
                                                    ? Qt.AscendingOrder : Qt.DescendingOrder)
                    for (var i = 0; i < popupListView.count; ++i) {
                        if (i != popupListView.currentIndex)
                            popupListView.model.setProperty(i, "order", Qt.AscendingOrder)
                    }

                    var isRecycle = titleBar.folderType === InteractionManager.RECYCLEROOT ? true : false

                    interactionManager.setSortType(model.type, model.order, isRecycle)

                    titleBar.listViewSortSignal()
                }
            }
        }
    }
    Component.onCompleted: {
        recycleSortType = interactionManager.getNoteSortModel(true)
        noteSortType = interactionManager.getNoteSortModel(false)
        interactionManager.sigRecycleSortTypeChanged.connect(setRecycleSortType)
        interactionManager.sigNoteSortTypeChanged.connect(setNoteSortType)
    }

    function setRecycleSortType(type) {
        recycleSortType = type
    }

    function setNoteSortType(type) {
        noteSortType = type
    }

}

