import QtQuick 2.11
import Qt.labs.platform 1.0
import QtQuick.Window 2.11
import QtQuick.Layouts 1.11
import QtQuick.Controls 2.4
import QtQuick.Dialogs 1.0
import NoteBacked 1.0
import org.deepin.dtk 1.0
import org.deepin.dtk.settings 1.0 as Settings

TitleBar {
    id: titleBar

    //当前选中的树节点（回收站、笔记本、快速笔记）
    property int folderType: InteractionManager.QUICKNOTEROOT
    //当前是否是默认页面（是 不显示视图切换按钮）
    property bool defaultPage: false
    //当前的视图页面
    property int viewMode: InteractionManager.ThreeColumn
    //快速笔记排序当前默认选中的节点（记录在文件）
    property int quickDefaultIndex: 0
    //回收站排序当前默认选中的节点（记录在文件）
    property int recycleDefaultIndex: 0
    //笔记本排序当前默认选中的节点（记录在文件）
    property int noteDefaultIndex: 0
    //当前是否显示工具栏
    property bool showCommonBar: false


    // 切换页面视图
    signal mainViewSwitchSignal(int iMainWindowViewModel)
    // 切换笔记列表视图
    signal listViewSwitchSignal(int iNoteListViewModel)
    //笔记排序
    signal listViewSortSignal()
    // 开启关闭工具栏
    signal showCommonBarSignal(bool isShow)
    // 插入图片通知
    signal insertImageSignal(var listData)
    //清空搜索栏
    signal clearSearchEdit()
    //搜索信号
    signal searchSignal(string key, int searchType)
    // 开始录音
    signal startRecordSignal()
    //开始搜索
    signal startSearchSignal()

    icon.name: "deepin-note"

    //    MouseArea {
    //        id: titleMouse
    //        anchors.fill: parent
    //        acceptedButtons: Qt.RightButton | Qt.LeftButton
    //        propagateComposedEvents: true
    //        onPressed: {
    //            mouse.accepted = false
    //            titleBar.focus = true
    //        }
    //        onClicked: {
    //            mouse.accepted = false
    //            titleBar.focus = true
    //        }
    //    }

    leftContent: ToolButton {
        id: switchViewButton
        icon.name: "sidebar"
        width: 54
        height: 36
        indicator: ButtonIndicator { }
        onClicked: { viewSwitchMenu.open() }

        //主页面视图切换菜单
        Menu {
            id: viewSwitchMenu
            x: switchViewButton.x
            y: titleBar.height - 8
            modal: true

            MenuItem {
                checkable: true
                icon.name: "three-column"
                autoExclusive: true
                text: qsTr("Three-column mode")
                display: IconLabel.IconBesideText
                checked: InteractionManager.ThreeColumn === titleBar.viewMode
                onClicked: {
                    setViewMode(InteractionManager.ThreeColumn)
                    titleBar.mainViewSwitchSignal(titleBar.viewMode)
                }
            }

            MenuItem {
                checkable: true
                icon.name: "two-column"
                autoExclusive: true
                text: qsTr("Two-column mode")
                display: IconLabel.IconBesideText
                checked: InteractionManager.TwoColumn === titleBar.viewMode
                onClicked: {
                    setViewMode(InteractionManager.TwoColumn)
                    titleBar.mainViewSwitchSignal(titleBar.viewMode)
                }
            }

            MenuItem {
                checkable: true
                icon.name: "Focus"
                autoExclusive: true
                text: qsTr("Focus mode")
                display: IconLabel.IconBesideText
                checked: InteractionManager.FocusColumn === titleBar.viewMode
                onClicked: {
                    setViewMode(InteractionManager.FocusColumn)
                    titleBar.mainViewSwitchSignal(titleBar.viewMode)
                }
            }
        }
    }

    content: RowLayout {
        anchors {
            left: parent.left
            right: parent.right
            leftMargin: 10
            rightMargin: 10
        }

        ToolButton {
            id: switchListButton
            icon.name: "thumbnail"
            width: 54
            height: 36
            indicator: ButtonIndicator { }
            visible: !(titleBar.defaultPage || titleBar.viewMode === InteractionManager.FocusColumn)
            onClicked: {
                if (popupLoder.status === Loader.Null) {
                    popupLoder.source = "SortPopup.qml"
                }
                if (titleBar.folderType !== InteractionManager.RECYCLEROOT) {
                    popupLoder.item.popupModel= popupLoder.noteModel
                } else {
                    popupLoder.item.popupModel = popupLoder.recycleModel
                }
                popupLoder.item.open()
            }
        }

        /*笔记本弹窗*/
        Loader {
            id: popupLoder
            property var noteModel
            property var recycleModel
            onLoaded: {
                var noteComponent = Qt.createComponent("SortModel.qml")
                noteModel = noteComponent.createObject(popupLoder
                                                       , {currentModel: InteractionManager.NOTEBOOKNORMAL});

                var recylceComponent = Qt.createComponent("SortModel.qml")
                recycleModel = recylceComponent.createObject(popupLoder
                                                             , {currentModel: InteractionManager.RECYCLEROOT});
            }
        }


        NoteSearchEdit{
            width: 350
            height: 36
            Layout.alignment: Qt.AlignCenter
            enabled: !titleBar.defaultPage
        }


        Row{
            id: rightbt
            spacing: 10
            Layout.alignment: Qt.AlignRight
            enabled: !titleBar.defaultPage
            visible: !(titleBar.folderType === InteractionManager.RECYCLEROOT)
            ToolButton {
                id: rtfbt
                width: 36
                height: 36
                icon.name: "rtf"
                icon.height : 36
                icon.width : 36
                onClicked:{
                    if(showCommonBar){
                        showCommonBar = false
                    }else{
                        showCommonBar = true
                    }
                    rtfbt.highlighted = showCommonBar
                    titleBar.showCommonBarSignal(showCommonBar)
                }
            }
            ToolButton {
                id: imgbt
                width: 36
                height: 36
                icon.name: "pic"
                icon.height : 36
                icon.width : 36
                onClicked:{
                    folderDialog.open();
                }
            }
            ToolButton {
                id: insertbt
                width: 36
                height: 36
                icon.name: "insert"
                icon.height : 36
                icon.width : 36
                onClicked:{
                    insertMenu.open();
                }
            }
        }

        //录音表格菜单
        Menu {
            id: insertMenu
            x: rightbt.x
            y: titleBar.height - 8
            width: 150;
            property bool enabledRecord: false
            MenuItem {
                icon.name: "recording"
                text: qsTr("Record audio")
                enabled: insertMenu.enabledRecord
                onTriggered:{
                    startRecordSignal();
                }
            }
            MenuItem {
                icon.name: "table"
                text: qsTr("Table")
                onTriggered:{
                }
            }
            Component.onCompleted: {
                insertMenu.enabledRecord = stateManager.getState(StatesManager.StateInputDevice)
                stateManager.sigInputDeviceChanged.connect(function(flag){
                    insertMenu.enabledRecord = flag
                })
            }
        }

        FileDialog {
            id: folderDialog
            title: "打开图片文件"
            nameFilters: ["HTML files (*.png *.jpg *.bmp)"]
            selectMultiple: true
            onAccepted: {
                titleBar.insertImageSignal(fileUrls);
            }
        }


    }

    menu: Menu{
        x: 0;
        y: 0;
        MenuItem{
            text : qsTr("Settings")
            onTriggered:{
                var component = Qt.createComponent("SettingsDialog.qml")
                var settingsDialog = component.createObject(titleBar)
                settingsDialog.show()
            }
        }

        ThemeMenu {}
        MenuSeparator {}
        HelpAction {}
        AboutAction {
            aboutDialog: AboutDialog {
                modality: Qt.NonModal
                version: qsTr(String("Version: %1").arg(Qt.application.version))
            }
        }
        QuitAction {}
    }

    /* 设置当前的笔记类型*/
    function setFolderType(type) {
        if (type !== titleBar.folderType) {
            titleBar.folderType = type
            titleBar.clearSearchEdit()
        }
    }

    /* 设置当前是否是缺省页面 */
    function setDefaultPage(isDefaultPage, isRecycle) {
        titleBar.defaultPage = isDefaultPage
    }

    /* 设置当前视图模式 */
    function setViewMode(mode) {
        titleBar.viewMode = mode
    }

}
