/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     guoshaoyu <guoshaoyu@uniontech.com>
*
* Maintainer: guoshaoyu <guoshaoyu@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "interactionmanager.h"
#include <QDebug>

InteractionManager::InteractionManager(QObject *parent) : QObject(parent)
{
    m_currentViewType = NoteListViewModel::AbstractView;
    m_noteSortType.sortModel = NoteListViewSortModel::ModifyTime;
    m_recycleSortType.sortModel = NoteListViewSortModel::DeteleTime;

    m_settings = new NoteSetting(parent);
}

InteractionManager *InteractionManager::instance()
{
    static InteractionManager ins;
    return &ins;
}

void InteractionManager::SetViewType(NoteListViewModel parType)
{
    m_currentViewType = parType;
}

int InteractionManager::GetViewType()
{
    return m_currentViewType;
}

void InteractionManager::setSortType(int parSort, int isInverted, bool isRecycle)
{
    if (isRecycle) {
        m_recycleSortType.sortModel = static_cast<NoteListViewSortModel>(parSort);
        sigRecycleSortTypeChanged(m_recycleSortType.sortModel);
        m_recycleSortType.bIsInverted = static_cast<Qt::SortOrder>(isInverted);
    } else {
        m_noteSortType.sortModel = static_cast<NoteListViewSortModel>(parSort);
        sigNoteSortTypeChanged(m_noteSortType.sortModel);
        m_noteSortType.bIsInverted = static_cast<Qt::SortOrder>(isInverted);
    }
}

InteractionManager::NoteSortType InteractionManager::getSortType(bool isRecycle)
{
    if (isRecycle) {
        return m_recycleSortType;
    } else {
        return m_noteSortType;
    }
}

int InteractionManager::getNoteSortModel(bool isRecycle)
{
    NoteSortType noteSortType = getSortType(isRecycle);
    return noteSortType.sortModel;
}

void InteractionManager::syncToSettings()
{
    m_settings->sync();
}

QVariant InteractionManager::valueFromSettings(const QString &key)
{
    return m_settings->value(key);
}

void InteractionManager::setValueToSettings(const QString &key, const QVariant &value)
{
    m_settings->setValue(key, value);
}

void InteractionManager::previewShortcut(QPointF pos)
{
        QJsonObject shortcutObj;
        QJsonArray jsonGroups;

        //noteBooks
        QMap<QString, QString> shortcutNotebookKeymap = {
            //Notebook
            {tr("New notebook"), "Ctrl+N"},
            {tr("Rename notebook"), "F2"},
            {tr("Delete notebook"), "Delete"},
        };

        QJsonObject notebookJsonGroup;
        notebookJsonGroup.insert("groupName", tr("Notebooks"));
        QJsonArray notebookJsonItems;

        for (QMap<QString, QString>::iterator it = shortcutNotebookKeymap.begin();
             it != shortcutNotebookKeymap.end(); ++it) {
            QJsonObject jsonItem;
            jsonItem.insert("name", it.key());
            jsonItem.insert("value", it.value().replace("Meta", "Super"));
            notebookJsonItems.append(jsonItem);
        }

        notebookJsonGroup.insert("groupItems", notebookJsonItems);
        jsonGroups.append(notebookJsonGroup);

        //notes
        QMap<QString, QString> shortcutNoteKeymap = {
            //Note
            {tr("New note"), "Ctrl+B"},
            {tr("Rename note"), "F3"},
            {tr("Delete note"), "Delete"},
            {tr("Play/Pause"), "Space"},
            {tr("Record voice"), "Ctrl+R"},
            {tr("Save note"), "Ctrl+S"},
            {tr("Save recordings"), "Ctrl+Y"},
            {tr("Find"), "Ctrl+F"},
        };

        QJsonObject noteJsonGroup;
        noteJsonGroup.insert("groupName", tr("Notes"));
        QJsonArray noteJsonItems;

        for (QMap<QString, QString>::iterator it = shortcutNoteKeymap.begin();
             it != shortcutNoteKeymap.end(); ++it) {
            QJsonObject jsonItem;
            jsonItem.insert("name", it.key());
            jsonItem.insert("value", it.value().replace("Meta", "Super"));
            noteJsonItems.append(jsonItem);
        }

        noteJsonGroup.insert("groupItems", noteJsonItems);
        jsonGroups.append(noteJsonGroup);

        //edit
        QMap<QString, QString> shortcutEditKeymap = {
            //Edit
            {tr("Select all"), "Ctrl+A"},
            {tr("Copy"), "Ctrl+C"},
            {tr("Cut"), "Ctrl+X"},
            {tr("Paste"), "Ctrl+V"},
            {tr("Delete"), "Delete"},
            {tr("redo"), "Ctrl+Z"},
            {tr("undo"), "Ctrl+Shift+Z"}
        };

        QJsonObject editJsonGroup;
        editJsonGroup.insert("groupName", tr("Edit"));
        QJsonArray editJsonItems;

        for (QMap<QString, QString>::iterator it = shortcutEditKeymap.begin();
             it != shortcutEditKeymap.end(); ++it) {
            QJsonObject jsonItem;
            jsonItem.insert("name", it.key());
            jsonItem.insert("value", it.value().replace("Meta", "Super"));
            editJsonItems.append(jsonItem);
        }

        editJsonGroup.insert("groupItems", editJsonItems);
        jsonGroups.append(editJsonGroup);

        //settings
        QMap<QString, QString> shortcutSettingKeymap = {
            //Setting
            {tr("Help"), "F1"},
            {tr("Display shortcuts"), "Ctrl+Shift+?"},
        };

        QJsonObject settingJsonGroup;
        settingJsonGroup.insert("groupName", tr("Settings"));
        QJsonArray settingJsonItems;

        for (QMap<QString, QString>::iterator it = shortcutSettingKeymap.begin();
             it != shortcutSettingKeymap.end(); ++it) {
            QJsonObject jsonItem;
            jsonItem.insert("name", it.key());
            jsonItem.insert("value", it.value().replace("Meta", "Super"));
            settingJsonItems.append(jsonItem);
        }

        settingJsonGroup.insert("groupItems", settingJsonItems);
        jsonGroups.append(settingJsonGroup);

        shortcutObj.insert("shortcut", jsonGroups);

        QJsonDocument doc(shortcutObj);

        QStringList shortcutString;
        QString param1 = "-j=" + QString(doc.toJson().data());
        QString param2 = "-p=" + QString::number(pos.x()) + "," + QString::number(pos.y());
        shortcutString << param1 << param2;

        QProcess *shortcutViewProcess = new QProcess(this);
        shortcutViewProcess->startDetached("deepin-shortcut-viewer", shortcutString);

        connect(shortcutViewProcess, SIGNAL(finished(int)), shortcutViewProcess, SLOT(deleteLater()));
}
