/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     guoshaoyu <guoshaoyu@uniontech.com>
*
* Maintainer: guoshaoyu <guoshaoyu@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef INTERACTIONMANAGER_H
#define INTERACTIONMANAGER_H

#include <QObject>
#include <QPoint>
#include <QProcess>
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonDocument>

#include "notesetting.h"

class InteractionManager : public QObject
{
    Q_OBJECT
public:
    //explicit InteractionManager(QObject *parent = nullptr);

    /* 主页面视图显示模式 */
    enum MainWindowViewModel {
        ThreeColumn = 0, //三栏模式
        TwoColumn,       //双栏模式
        FocusColumn      //专注模式
    };
    Q_ENUM(MainWindowViewModel)

    /* 笔记视图显示模式 */
    enum NoteListViewModel {
        ListView = 0,    //列表模式
        AbstractView,    //摘要模式
        CardView         //卡片模式
    };
    Q_ENUM(NoteListViewModel)

    /* 笔记视图排序模式 */
    enum NoteListViewSortModel {
        ModifyTime = 0,  //修改时间
        CreationTime,    //创建时间
        NoteName,        //笔记名称
        Customize,       //自定义 笔记列表独有
        DeteleTime,  //删除时间 回收站独有
    };
    Q_ENUM(NoteListViewSortModel)

    /* 笔记类型 */
    enum FolderType {
        NOTEBOOKROOT = 0, //笔记本根节点
        QUICKNOTEROOT, //快速笔记根节点
        RECYCLEROOT, //回收站根节点
        NOTEBOOKNORMAL //普通笔记本
    };
    Q_ENUM(FolderType)

    /* 弹窗类型 */
    enum DialogType {
        ClearRecycle = 0,       //清空回收站
        MultipleDeleteNotes,    //多选删除
        PermanentlyDeleteNotes, //永久删除
        DeleteNotebook,         //删除笔记本
        CreatNotebook,          //创建笔记本
        CreatMulNotebook,       //创建嵌套笔记本
        ModifyNotebook          //修改笔记本
    };
    Q_ENUM(DialogType)

    struct NoteSortType {
        NoteListViewSortModel sortModel{ NoteListViewSortModel::ModifyTime }; //排序规则

        Qt::SortOrder bIsInverted { Qt::DescendingOrder };
    };

    static InteractionManager *instance();

public:
    /**
     * @brief SetViewType 设置当前视图模式
     * @param parType 视图模式
     */
    Q_INVOKABLE void SetViewType(NoteListViewModel parType);

    /**
     * @brief GetViewType 获取当前视图模式
     * @param
     */
    Q_INVOKABLE int GetViewType();

    /**
     * @brief SetSortType 设置当前排序规则
     * @param parSort 模式
     * @param bIsInverted 是否为倒序
     * @param isRecycle 是否为回收站类型
     */
    Q_INVOKABLE void setSortType(int parSort, int isInverted, bool isRecycle);

    /**
     * @brief GetSortType 获取排序规则
     * @param isRecycle 是否为回收站类型
     */
    Q_INVOKABLE NoteSortType getSortType(bool isRecycle);

    /**
     * @brief getNoteSortModel 获取排序模式
     * @param isRecycle 是否为回收站类型
     * @return 0:修改时间,1:创建时间,2:笔记名称,3:自定义
     */
    Q_INVOKABLE int getNoteSortModel(bool isRecycle);

    //配置文件
    void syncToSettings();
    QVariant valueFromSettings(const QString &key);
    void setValueToSettings(const QString &key, const QVariant &value);

    /**
     * @brief PreviewShortcut 显示快捷键
     */
    Q_INVOKABLE void previewShortcut(QPointF pos);

signals:
    /**
     * @brief sigRecycleSortTypeChanged 回收站排序改变信号
     * @param type
     */
    void sigRecycleSortTypeChanged(int type);
    /**
     * @brief sigNoteSortTypeChanged 笔记排序改变信号
     * @param type
     */
    void sigNoteSortTypeChanged(int type);

protected:
    explicit InteractionManager(QObject *parent = nullptr);

private:

    //当前视图模式
    NoteListViewModel m_currentViewType;

    //笔记本排序规则
    NoteSortType m_noteSortType;

    //回收站排序规则
    NoteSortType m_recycleSortType;

    //设置
    NoteSetting *m_settings;
};

#endif // INTERACTIONMANAGER_H
