/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NOTEDATA_H
#define NOTEDATA_H

#include <QDateTime>
#include <QMetaType>

enum noteContentType {
    NOTEJSONTEXT, //json数据
    NOTEHTMLTEXT, //html数据
    NOTEMARKDOWNTEXT, //markdown数据
};

enum FolderContentType {
    NOTEBOOKROOT, //笔记本根节点
    QUICKNOTEROOT, //快速笔记根节点
    RECYCLEROOT, //回收站根节点
    NOTEBOOKNORMAL, //普通笔记本
};

struct ContentBase {
    bool deleteFlag {false}; //删除标志，默认0,1代表删除进入回收站
    qint64 id {-1}; //唯一标识
    qint64 parentId {-1};//父节点id
    qint64 customSortNum {-1}; //自定义排序序号
    QString name {""}; //名称
    QString password {""}; //密码hash值
    QDateTime createTime; //创建时间
    QDateTime modifyTime; //修改时间
    QDateTime deleteTime; //删除时间
};

struct NoteContent : ContentBase {
    noteContentType type {NOTEHTMLTEXT}; //笔记类型
    bool top {false}; //笔记置顶
    qint64 fontSize {0};
    QString fontName{""}; //字体名称
    QString synopsisContent {""}; //内容摘要
    QString synopsisIcon {""}; //内容摘要图片
    QString content {""}; //笔记内容
    QStringList tags{""}; //标签
    QDateTime topTime; //置顶时间
};

struct FolderContent: ContentBase {
    FolderContentType type; //类型
    QString iconUrl; //头像路径
    QString children {""};
};

struct VoiceInfo {
    QDateTime createTime;
    QString text{""};
    QString title {""};
    QString voicePath {""};
    qint64 voiceSize{0};
};

Q_DECLARE_METATYPE(FolderContent);

#endif // NOTEDATA_H
