/*
 * Copyright (C) 2022 Uniontech Software Technology Co., Ltd.
 *
 * Author:     zhangzhong <zhangzhong@uniontech.com>
 *
 * Maintainer: zhangzhong <zhangzhong@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "notesetting.h"

#include <DSettingsOption>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <qsettingbackend.h>
#include <QCoreApplication>
#include <QStandardPaths>

NoteSetting::NoteSetting(QObject *parent) : QObject(parent)
{
    init();
}

NoteSetting::~NoteSetting()
{
    if (m_settings != nullptr) {
        m_settings->sync();
        delete m_settings;
        m_settings = nullptr;
    }
}

void NoteSetting::init()
{
    QString vnoteConfigBasePath =
        QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
    QFileInfo configDir(vnoteConfigBasePath + QDir::separator());
    if (!configDir.exists()) {
        QDir().mkpath(configDir.filePath());
    }

    auto configFilepath = vnoteConfigBasePath + "/noteconfig.ini";
    m_settings = Dtk::Core::DSettings::fromJsonFile(":/note-settings.json");
    auto backend = new Dtk::Core::QSettingBackend(configFilepath, m_settings);
    m_settings->setBackend(backend);
}

QPointer<Dtk::Core::DSettings> NoteSetting::settings()
{
    return m_settings;
}

void NoteSetting::sync()
{
    if (!m_settings.isNull())
        m_settings->sync();
}

QVariant NoteSetting::value(const QString &key)
{
    return m_settings.isNull() ? QVariant() : m_settings->value(key);
}

void NoteSetting::setValue(const QString &key, const QVariant &value)
{
    if (!m_settings.isNull())
        m_settings->setOption(key, value);
}
