/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "statesmanager.h"

StatesManager::StatesManager(QObject *parent) : QObject(parent)
{
    m_audioWatcher = AudioWatcher::instance();
    slotAudioDeviceChanged(AudioWatcher::Micphone);
    slotAudioDeviceChanged(AudioWatcher::Internal);
    connect(m_audioWatcher, &AudioWatcher::sigDeviceChange, this, &StatesManager::slotAudioDeviceChanged);
    connect(m_audioWatcher, &AudioWatcher::sigVolumeChange, this, &StatesManager::slotAudioVolumeChanged);
}
StatesManager *StatesManager::instance()
{
    static StatesManager ins;
    return  &ins;
}
void StatesManager::setState(State type, bool flag)
{
    if (getState(type) != flag) {
        m_states.setFlag(type, flag);
        switch (type) {
        case StateSearching:
            emit sigStateSearchChanged(flag);
            break;
        case StateRecording:
            emit sigStateRecordChanged(flag);
            break;
        case StatePlaying:
            emit sigStatePlayChanged(flag);
            break;
        case StateVoice2Text:
            emit sigStateVoice2TextChanged(flag);
            break;
        case StateInputDevice:
            emit sigInputDeviceChanged(flag);
            break;
        case StateOutputDevice:
            emit sigOutputDeviceChanged(flag);
            break;
        default:
            break;
        }
    }
}

bool StatesManager::getState(State type)
{
    return m_states.testFlag(type);
}

void StatesManager::slotAudioDeviceChanged(AudioWatcher::AudioMode mode)
{
    State type = StateInputDevice;
    if (AudioWatcher::Internal == mode) {
        type = StateOutputDevice;
    }
    setState(type, !m_audioWatcher->getDeviceName(mode).isEmpty());
}

void StatesManager::slotAudioVolumeChanged(AudioWatcher::AudioMode mode)
{
    if (AudioWatcher::Micphone == mode) {
        emit sigInputDeviceVolumeChanged(m_audioWatcher->getVolume(mode));
    } else {
        emit sigOutputDeviceVolumeChanged(m_audioWatcher->getVolume(mode));
    }
}
