/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef OPSSTATEINTERFACE_H
#define OPSSTATEINTERFACE_H
#include "audiowatcher.h"

#include <QObject>
#include <QFlags>

class StatesManager : public QObject
{
    Q_OBJECT
public:
    enum State {
        StateSearching = 0x01,
        StateRecording = 0x02,
        StatePlaying = 0x04,
        StateVoice2Text = 0x08,
        StateAISrvAvailable = 0x10,
        StateAppQuit = 0x20,
        StateInputDevice = 0x40,
        StateOutputDevice = 0x80
    };
    Q_ENUM(State)
    Q_DECLARE_FLAGS(States, State)
    static StatesManager *instance();
signals:
    /**
     * @brief sigActiveWindow 激活窗口
     */
    void sigActiveWindow();
    void sigStateSearchChanged(bool flag);
    void sigStateRecordChanged(bool flag);
    void sigStatePlayChanged(bool flag);
    void sigStateVoice2TextChanged(bool flag);
    void sigInputDeviceChanged(bool flag);
    void sigOutputDeviceChanged(bool flag);
    void sigInputDeviceVolumeChanged(double Volume);
    void sigOutputDeviceVolumeChanged(double Volume);
public slots:
    /**
     * @brief setState 设置状态
     * @param type 类型
     * @param flag 标志
     */
    void setState(State type, bool flag);
    /**
     * @brief getState 获取状态
     * @param type 类型
     * @return 状态标志
     */
    bool getState(State type);
    /**
     * @brief slotAudioDeviceChanged 录音设备变化
     * @param mode
     */
    void slotAudioDeviceChanged(AudioWatcher::AudioMode mode);
    /**
     * @brief slotAudioVolumeChanged 录音设备音量变化
     * @param mode
     */
    void slotAudioVolumeChanged(AudioWatcher::AudioMode mode);
protected:
    explicit StatesManager(QObject *parent = nullptr);
private:
    States m_states;
    AudioWatcher *m_audioWatcher {nullptr};
};

#endif // OPSSTATEINTERFACE_H
