/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef UTILS_H
#define UTILS_H

#include "notedata.h"

#include <QObject>
#include <QAudioBuffer>

class Utils : public QObject
{
    Q_OBJECT
public:
    ~Utils();
    /**
     * @brief getInstance 单例
     * @return 单例对象
     */
    static Utils *getInstance();
    /**
     * @brief mkMutiDir 创建多级文件夹
     * @param path 文件夹路径
     * @return 创建路径
     */
    QString mkMutiDir(const QString &path);
    /**
     * @brief parseVoiceInfo 解析语音信息
     * @param jsonData json串
     * @param info 输出结构体
     * @return
     */
    bool parseVoiceInfo(const QString &jsonData, VoiceInfo &info);
    /**
     * @brief makeVoiceInfo 语音信息生成json串
     * @param info 语音信息
     * @return json串
     */
    QString makeVoiceInfo(const VoiceInfo &info);
    /**
     * @brief getAudioWaveform 获取语音波形
     * @param data 数据
     * @param offset 数据偏移，默认0
     * @return
     */
    qreal getAudioWaveform(const QAudioBuffer &data, qint64 offset = 0);

    /**
     * @brief getAudioSavePath 获取语音保存路径
     * @return
     */
    QString getAudioSavePath();
protected:
    explicit Utils(QObject *parent = nullptr);
};

#endif // UTILS_H
