/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuxinping <liuxinping@uniontech.com>
*
* Maintainer: liuxinping <liuxinping@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LEFTTREEMODE_H
#define LEFTTREEMODE_H
#include "common/notedata.h"

#include <QStandardItemModel>
#include <QSortFilterProxyModel>
class LeftTreeMode : public QSortFilterProxyModel
//class LeftTreeMode : public QStandardItemModel
{
    Q_OBJECT
public:
    explicit LeftTreeMode(QObject *parent = nullptr);
    ~LeftTreeMode() Q_DECL_OVERRIDE;

    bool lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const Q_DECL_OVERRIDE;

    /**
     * @brief createStandardItem 新建item 数据
     * @param data
     * @return
     */
    QStandardItem *createStandardItem(void *data);

    /**
     * @brief getStandardItemData 根据index 获取数据
     * @param index
     * @return
     */
    FolderContent *getStandardItemData(const QModelIndex &index) const;

public slots:

    /**
     * @brief slotaddNewNoteBook 新建笔记本按钮函数
     */
    void slotaddNewNoteBook(const QString &text);

    /**
     * @brief slotAddChilderNode 根据 index 添加分组
     * @param index
     */
    void slotAddChilderNode(const QModelIndex &index, const QString &text);

    /**
     * @brief slotLoadDataFinsh 数据加载完成通知
     */
    void slotLoadDataFinsh();

    /**
     * @brief slotGetItemName 获取名称
     * @param index
     * @return
     */
    QString slotGetItemName(const QModelIndex &index);

    /**
     * @brief slotGetItemIcon 获取图片
     * @param index
     * @return
     */
    QString slotGetItemIcon(const QModelIndex &index);

    /**
     * @brief slotGetItemCount 获取当前项的笔记个数
     * @param index
     * @return
     */
    QString slotGetItemCount(const QModelIndex &index);

    /**
     * @brief slotGetItemFolderId 获取当前选中项的id
     * @param index
     * @return
     */
    qint64 slotGetItemFolderId(const QModelIndex &index);

    /**
     * @brief slotRenameNoteBooks 重命名
     * @param index
     * @param text
     */
    void slotRenameNoteBooks(const QModelIndex &index, const QString &text);

    /**
     * @brief slotDeleteNoteBookFolder 删除笔记本
     * @param index
     * @return 删除是否成功
     */
    void slotDeleteNoteBookFolder(const QModelIndex &index);

    /**
     * @brief slotupdateData
     */
    void slotupdateData();

signals:

    /**
     * @brief signalDeleteNoticeToQml 删除笔记本时发送index通知qml treeview 设置选中
     * @param index
     */
    void signalDeleteNoticeToQml(QModelIndex index);

    /**
     * @brief signalCreateNoticeToQml 新建笔记本时发送index通知qml treeview 设置选中
     * @param index
     */
    void signalCreateNoticeToQml(QModelIndex index);

private:
    /**
     * @brief loadData 加载数据库数据
     */
    void loadData();

    /**
     * @brief initData 初始化数据
     */
    void initUiData();

    /**
     * @brief initDbData 递归加载数据库数据
     * @param data
     * @param folderP
     */
    void initDbData(QStandardItem *data, FolderContent *folderP);

    /**
     * @brief deleteData 递归删除笔记本
     * @param data
     * @param row
     * @param folderP
     */
    void deleteData(QStandardItem *data, int row, FolderContent *folderP, QStringList &iter);
private:
    QStandardItem       *m_pQuickNoteItem {nullptr};    //快速笔记根节点item
    QStandardItem       *m_pNoteBooksItem {nullptr};    //笔记本根节点item
    QStandardItemModel  *m_pModel {nullptr};
};

#endif // LEFTTREEMODE_H
