/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DBMANAGER_H
#define DBMANAGER_H
#include <functional>

#include <QObject>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QMutex>

class DbManager : public QObject
{
    Q_OBJECT
public:
    ~DbManager();
    /**
     * @brief getInstance 单例
     * @return 单例对象
     */
    static DbManager *getInstance();
    /**
     * @brief operationDb
     * @param sql sql 语句
     * @param resultCallback 结果回调函数
     * @return 结果，成功true,失败false
     */
    bool operationDb(const QString &sql, std::function<void (QSqlQuery &)> resultCallback = nullptr);

    /**
     * @brief checkSqlStr 检查sql语句
     * @param sql
     */
    void checkSqlStr(QString &sql);

protected:
    explicit DbManager(QObject *parent = nullptr);
    /**
     * @brief initDB 初始化数据库
     * @return 成功 true
     */
    bool initDB();
private:
    QSqlDatabase m_dbbase;
    QMutex m_dbLock;
    bool m_isDbInitOK = {false};
};

#endif // DBMANAGER_H
