/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "folderdboper.h"
#include "notedata.h"
#include "dbmanager.h"

#include <QDebug>

FolderDbOper::FolderDbOper(QObject *parent) : QObject(parent)
{

}

bool FolderDbOper::addFolder(FolderContent *data)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = "";
    QString tempName = data->name;
    DbManager::getInstance()->checkSqlStr(tempName);
    sql.sprintf("INSERT INTO note_folder(type,parent,name,icon,create_time,modify_time) VALUES(%d, %lld,'%s','%s',%u,%u)",
                data->type,
                data->parentId,
                tempName.toUtf8().data(),

                data->iconUrl.toUtf8().data(),
                data->createTime.toTime_t(),
                data->modifyTime.toTime_t());
    if (DbManager::getInstance()->operationDb(sql)) {
        return DbManager::getInstance()->operationDb("SELECT seq from sqlite_sequence WHERE name='note_folder'", [data](QSqlQuery & result) {
            if (result.next()) {
                data->id = result.value("seq").toLongLong();
            }
        });
    }
    return false;
}

bool FolderDbOper::deleteFolder(const QStringList &data)
{
    if (data.isEmpty()) {
        return false;
    }
    QString idlist = data.join(",");
    QString sql = QString("DELETE FROM note_folder WHERE id in (%1) ").arg(idlist);

    if (!DbManager::getInstance()->operationDb(sql)) {
        return false;
    }
    return true;
}

bool FolderDbOper::updateName(FolderContent *data, const QString &newName)
{
    if (nullptr == data) {
        return false;
    }
    QString tempName = newName;
    DbManager::getInstance()->checkSqlStr(tempName);
    QDateTime time = QDateTime::currentDateTime();
    QString sql = QString().sprintf("UPDATE note_folder SET name='%s',modify_time=%u WHERE id=%lld", tempName.toUtf8().data(), time.toTime_t(), data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->name = newName;
        data->modifyTime = time;
        return true;
    }
    return false;
}

bool FolderDbOper::updateChildren(FolderContent *data, const QString &newChildren)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = QString().sprintf("UPDATE note_folder SET children='%s' WHERE id=%lld", newChildren.toUtf8().data(), data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->children = newChildren;
        return true;
    }
    return false;
}

bool FolderDbOper::updateParent(FolderContent *data, qint64 parentid)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = QString().sprintf("UPDATE note_folder SET parent=%lld WHERE id=%lld", parentid, data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->parentId = parentid;
        return true;
    }
    return false;
}

QHash<qint64, FolderContent *> FolderDbOper::loadFolders()
{
    QHash<qint64, FolderContent *> hash;
    static QString sql = "SELECT id,type,parent,delete_flag,name,children,icon,password_text,create_time,modify_time,delete_time from note_folder";
    DbManager::getInstance()->operationDb(sql, [&hash, this](QSqlQuery & result) {
        praseSqlQueryRestlt(hash, result);
    });

    return hash;
}

void FolderDbOper::praseSqlQueryRestlt(QHash<qint64, FolderContent *> &data, QSqlQuery &result)
{
    while (result.next()) {
        FolderContent *folder = new FolderContent;
        folder->id = result.value("id").toLongLong();
        folder->type = static_cast<FolderContentType>(result.value("type").toInt());
        folder->parentId = result.value("parent").toLongLong();
        folder->deleteFlag = result.value("delete_flag").toBool();
        folder->name = result.value("name").toString();
        folder->children = result.value("children").toString();
        folder->iconUrl = result.value("icon").toString();
        folder->password = result.value("password_text").toString();
        folder->createTime = QDateTime::fromTime_t(result.value("create_time").toUInt());
        folder->modifyTime = QDateTime::fromTime_t(result.value("modify_time").toUInt());
        folder->deleteTime = QDateTime::fromTime_t(result.value("delete_time").toUInt());
        data.insert(folder->id, folder);
    }
}
