/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef FOLDERDBOPER_H
#define FOLDERDBOPER_H

#include <QObject>
#include <QHash>
#include <QSqlQuery>

class FolderContent;

class FolderDbOper : public QObject
{
    Q_OBJECT
public:
    explicit FolderDbOper(QObject *parent = nullptr);
    /**
     * @brief addFolder 添加文件夹
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool addFolder(FolderContent *data);
    /**
     * @brief deleteFolder 删除文件夹
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool deleteFolder(const QStringList &data);

    /**
     * @brief updateName 更新名称
     * @param data 数据
     * @param newName 新名称
     * @return 结果，成功true,失败false
     */
    bool updateName(FolderContent *data, const QString &newName);

    /**
     * @brief updateChildren 更新子节点
     * @param data 数据
     * @param newChildren 子节点
     * @return 结果，成功true,失败false
     */
    bool updateChildren(FolderContent *data, const QString &newChildren);
    /**
     * @brief updateParent 更新父节点
     * @param data 数据
     * @param parentid 父节点
     * @return 结果，成功true,失败false
     */
    bool updateParent(FolderContent *data, qint64 parentid);
    /**
     * @brief loadFolders 加载所有文件夹
     * @return 结果，所有数据
     */
    QHash<qint64, FolderContent *> loadFolders();
private:
    /**
     * @brief praseSqlQueryRestlt 解析数据库查询结果
     * @param data 结构体
     * @param result 数据库返回的结果
     */
    void praseSqlQueryRestlt(QHash<qint64, FolderContent *> &data, QSqlQuery &result);
};

#endif // FOLDERDBOPER_H
