/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "notedboper.h"
#include "notedata.h"
#include "dbmanager.h"

#include <QDebug>

NoteDbOper::NoteDbOper(QObject *parent) : QObject(parent)
{

}

bool NoteDbOper::addNote(NoteContent *data)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = "";

    sql.sprintf("INSERT INTO note(type,parent,name,content,font_size,font_name,synopsis_content,synopsis_icon,create_time,modify_time) VALUES(%d,%lld,'%s','%s',%lld,'%s','%s','%s',%u,%u)",
                data->type,
                data->parentId,
                data->name.toUtf8().data(),
                data->content.toUtf8().data(),
                data->fontSize,
                data->fontName.toUtf8().data(),
                data->synopsisContent.toUtf8().data(),
                data->synopsisIcon.toUtf8().data(),
                data->createTime.toTime_t(),
                data->modifyTime.toTime_t());
    if (DbManager::getInstance()->operationDb(sql)) {
        return DbManager::getInstance()->operationDb("SELECT seq from sqlite_sequence WHERE name='note'", [data](QSqlQuery & result) {
            if (result.next()) {
                data->id = result.value("seq").toLongLong();
            }
        });
    }
    return false;
}

bool NoteDbOper::deleteNote(NoteContent *data, bool isRecycle)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = "";
    QDateTime time;
    if (isRecycle) {
        if (true == data->deleteFlag) {
            return true;
        }
        time = QDateTime::currentDateTime();
        sql.sprintf("UPDATE note SET delete_flag=1,delete_time=%u WHERE id=%lld", time.toTime_t(), data->id);
    } else {
        sql.sprintf("DELETE FROM note WHERE id=%lld", data->id);
    }

    if (!DbManager::getInstance()->operationDb(sql)) {
        return false;
    }

    if (!time.isNull()) {
        data->deleteTime = time;
        data->deleteFlag = true;
    }

    return true;
}

bool  NoteDbOper::updateName(NoteContent *data, const QString &newName)
{
    if (nullptr == data) {
        return false;
    }
    QDateTime time = QDateTime::currentDateTime();
    QString sql = QString().sprintf("UPDATE note SET name='%s',modify_time=%u WHERE id=%lld", newName.toUtf8().data(), time.toTime_t(), data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->name = newName;
        data->modifyTime = time;
        return true;
    }
    return false;
}

bool NoteDbOper::updateParent(NoteContent *data, qint64 parentid)
{
    if (nullptr == data) {
        return false;
    }
    QString sql = QString().sprintf("UPDATE note SET parent=%lld WHERE id=%lld", parentid, data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->parentId = parentid;
        return true;
    }
    return false;
}

bool NoteDbOper::updateContent(NoteContent *data, const QString &content, const QString &synopsis_content, const QString &synopsis_icon)
{
    if (nullptr == data) {
        return false;
    }

    QDateTime time = QDateTime::currentDateTime();
    QString tmpContent = content;
    DbManager::getInstance()->checkSqlStr(tmpContent);
    QString sql = QString("UPDATE note SET content='%1'").arg(tmpContent);
    if (data->synopsisContent != synopsis_content) {
        QString tmpSynopsisContent = synopsis_content;
        DbManager::getInstance()->checkSqlStr(tmpSynopsisContent);
        sql.append(QString(",synopsis_content='%1'").arg(tmpSynopsisContent));
    }
    if (data->synopsisIcon != synopsis_icon) {
        QString tmpSynopsisIcon = synopsis_icon;
        DbManager::getInstance()->checkSqlStr(tmpSynopsisIcon);
        sql.append(QString(",synopsis_icon='%1'").arg(tmpSynopsisIcon));
    }
    sql.append(QString(",modify_time=%1 WHERE id=%2").arg(time.toTime_t()).arg(data->id));
    if (DbManager::getInstance()->operationDb(sql)) {
        data->modifyTime = time;
        data->content = content;
        data->synopsisContent = synopsis_content;
        data->synopsisIcon = synopsis_icon;
        return true;
    }
    return false;
}

bool NoteDbOper::updateDeleteFlag(NoteContent *data, bool flag)
{
    if (nullptr == data) {
        return false;
    }
    if (data->deleteFlag == flag) {
        return true;
    }
    QString sql = QString().sprintf("UPDATE note SET delete_flag=%d WHERE id=%lld", flag, data->id);
    if (DbManager::getInstance()->operationDb(sql)) {
        data->deleteFlag = flag;
        return true;
    }
    return false;
}

bool NoteDbOper::updateTopFlag(NoteContent *data, bool flag)
{
    if (nullptr == data) {
        return false;
    }
    if (data->top == flag) {
        return true;
    }
    QString sql;
    QDateTime time;
    if (flag) {
        time = QDateTime::currentDateTime();
        sql = QString().sprintf("UPDATE note SET top=%d,top_time=%u WHERE id=%lld", flag, time.toTime_t(), data->id);
    } else {
        sql = QString().sprintf("UPDATE note SET top=%d WHERE id=%lld", flag, data->id);
    }
    if (DbManager::getInstance()->operationDb(sql)) {
        data->top = flag;
        if (time.isValid()) {
            data->topTime = time;
        }
        return true;
    }
    return false;
}

bool NoteDbOper::clearRecycle()
{
    QString sql = "DELETE FROM note WHERE delete_flag=true";
    if (!DbManager::getInstance()->operationDb(sql)) {
        return false;
    }
    return true;
}

bool NoteDbOper::deleteNoteList(const QStringList &parentId, QDateTime time)
{
    if (parentId.isEmpty()) {
        return false;
    }
    QString idlist = parentId.join(",");
    QString sql = QString("UPDATE note SET delete_flag=1,delete_time=%1 WHERE delete_flag=false and parent in (%2) ").arg(time.toTime_t()).arg(idlist);
    if (!DbManager::getInstance()->operationDb(sql)) {
        return false;
    }
    return true;
}

bool NoteDbOper::fullNoteContent(NoteContent *data)
{
    if (nullptr == data) {
        return false;
    }
    if (!data->content.isEmpty()) {
        return true;
    }

    QString sql = QString().sprintf("SELECT content from note WHERE id=%lld", data->id);
    return DbManager::getInstance()->operationDb(sql, [data](QSqlQuery & result) {
        if (result.next()) {
            data->content = result.value("content").toString();
        }
    });
}

QHash<qint64, NoteContent *> NoteDbOper::loadNotes()
{
    QHash<qint64, NoteContent *> hash;
    static QString sql = "SELECT id,parent,type,top,delete_flag,name,tags,password_text,font_size,font_name,synopsis_content,synopsis_icon,create_time,modify_time,delete_time,top_time from note";
    DbManager::getInstance()->operationDb(sql, [&hash, this](QSqlQuery & result) {
        praseSqlQueryRestlt(hash, result);
    });
    return hash;
}


void NoteDbOper::praseSqlQueryRestlt(QHash<qint64, NoteContent *> &data, QSqlQuery &result)
{
    while (result.next()) {
        NoteContent *note = new NoteContent;
        note->id = result.value("id").toLongLong();
        note->parentId = result.value("parent").toLongLong();
        note->type = static_cast<noteContentType>(result.value("type").toInt());
        note->top = result.value("top").toBool();
        note->deleteFlag = result.value("delete_flag").toBool();
        note->name = result.value("name").toString();
        note->tags = result.value("tags").toString().split(",");
        note->password = result.value("password_text").toString();
        note->fontSize = result.value("font_size").toInt();
        note->fontName = result.value("font_name").toString();
        note->synopsisContent = result.value("synopsis_content").toString();
        note->synopsisIcon = result.value("synopsis_icon").toString();
        note->createTime = QDateTime::fromTime_t(result.value("create_time").toUInt());
        note->modifyTime = QDateTime::fromTime_t(result.value("modify_time").toUInt());
        note->deleteTime = QDateTime::fromTime_t(result.value("delete_time").toUInt());
        note->topTime = QDateTime::fromTime_t(result.value("top_time").toUInt());
        data.insert(note->id, note);
    }
}
