/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NOTEDBOPER_H
#define NOTEDBOPER_H

#include <QObject>
#include <QHash>
#include <QSqlQuery>

class NoteContent;

class NoteDbOper : public QObject
{
    Q_OBJECT
public:
    explicit NoteDbOper(QObject *parent = nullptr);
    /**
     * @brief addNote 添加笔记
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool addNote(NoteContent *data);
    /**
     * @brief deleteNote 删除笔记
     * @param data 数据
     * @param isRecycle
     * @return 结果，成功true,失败false
     */
    bool deleteNote(NoteContent *data, bool isRecycle = true);
    /**
     * @brief updateName 更新名称
     * @param data 数据
     * @param newName 新名称
     * @return 结果，成功true,失败false
     */
    bool updateName(NoteContent *data, const QString &newName);
    /**
     * @brief updateParent 更新父节点
     * @param data 数据
     * @param parentid 父节点id
     * @return 结果，成功true,失败false
     */
    bool updateParent(NoteContent *data, qint64 parentid);
    /**
     * @brief updateContent 更新笔记内容
     * @param data 数据
     * @param content 内容
     * @param synopsis_content 内容摘要
     * @param synopsis_icon 内容摘要图标
     * @return 结果，成功true,失败false
     */
    bool updateContent(NoteContent *data, const QString &content, const QString &synopsis_content, const QString &synopsis_icon);
    /**
     * @brief updateDeleteFlag 更新删除标志
     * @param data 数据
     * @param flag 标志， true代表删除
     * @return 结果，成功true,失败false
     */
    bool updateDeleteFlag(NoteContent *data, bool flag);
    /**
     * @brief updateTopFlag 更新置顶标志
     * @param data 数据
     * @param flag 标志， true代表置顶
     * @return 结果，成功true,失败false
     */
    bool updateTopFlag(NoteContent *data, bool flag);
    /**
     * @brief fullNoteContent 加载笔记内容
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool fullNoteContent(NoteContent *data);
    /**
     * @brief loadNotes 加载笔记
     * @return 所有笔记
     */
    QHash<qint64, NoteContent *> loadNotes();

    /**
     * @brief clearRecycle 清空回收站
     * @return
     */
    bool clearRecycle();

    /**
     * @brief deleteNoteList 根据父id删除笔记本下多个笔记数据
     * @param parentId
     * @param time 删除时间
     * @return
     */
    bool deleteNoteList(const QStringList &parentId, QDateTime time);

private:
    /**
     * @brief praseSqlQueryRestlt 解析数据库查询结果
     * @param data 数据
     * @param result 数据库返回的结果
     */
    void praseSqlQueryRestlt(QHash<qint64, NoteContent *> &data, QSqlQuery &result);
};

#endif // NOTEDBOPER_H
