/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef AUDIOPLAYMANAGER_H
#define AUDIOPLAYMANAGER_H
#include "notedata.h"

#include <QObject>
#include <QQuickPaintedItem>
#include <QList>
#include <QHash>

class VlcPalyer;

class AudioPlayManager : public QQuickPaintedItem
{
    Q_OBJECT
    Q_PROPERTY(QString playTime READ getPlayTime WRITE setPlayTime NOTIFY sigPlayTimeChange)
    Q_PROPERTY(QString audioTitle READ getAudioTitle WRITE setAudioTitle NOTIFY sigAudioTitleChange)
public:
    explicit AudioPlayManager(QQuickItem *parent = nullptr);
    QString getPlayTime();
    void setPlayTime(const QString &time);
    QString getAudioTitle();
    void setAudioTitle(const QString &title);
signals:
    void sigPlayTimeChange();
    void sigAudioTitleChange();
    void sigPlayFinsh();
public slots:
    void slotDurationChanged(qint64 duration);
    void slotCurrentPosChanged(qint64 pos);
    int  slotStartPlay(const QString &data, bool isReset = true);
    void slotPlayFinsh();
    void slotPraseWaveformFinsh(const QString &url, const QList<qreal> &data);
protected:
    void paint(QPainter *painter) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
private:
    bool updatePlayTime();
private:
    bool m_sliderReleased {true};
    VoiceInfo m_audioInfo;
    qint64 m_currentPos {0};
    VlcPalyer *m_player {nullptr};
    QString m_playTime {"00:00/00:00"};
    QString m_strDuration {"00:00"};
    QHash<QString, QList<qreal>> m_audioScaleSamples;
};

#endif // AUDIOPLAYMANAGER_H
