/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef AUDIORECORDMANAGER_H
#define AUDIORECORDMANAGER_H

#include <QObject>
#include <QQuickPaintedItem>
#include <QAudioBuffer>
#include <QList>

class GstreamRecorder;
class AudioWatcher;

class AudioRecordManager : public QQuickPaintedItem
{
    Q_OBJECT
    Q_PROPERTY(QString recordTime READ getRecordTime WRITE setRecordTime NOTIFY sigRecordTimeChange)
public:
    explicit AudioRecordManager(QQuickItem *parent = nullptr);
    /**
     * @brief getRecordTime 获取录音时间
     * @return
     */
    QString getRecordTime();
    /**
     * @brief setRecordTime 设置录音时间
     * @param recordTime
     */
    void setRecordTime(const QString &recordTime);
signals:
    /**
     * @brief sigRecordFinsh 录音完成
     * @param data 语音数据
     */
    void sigRecordFinsh(const QString &data);
    /**
     * @brief sigRecordTimeChange 录音时间变化
     */
    void sigRecordTimeChange();
protected:
    void paint(QPainter *painter) override;
public slots:
    /**
     * @brief startRecord 开始录音
     * @return
     */
    bool slotStartRecord();
    /**
     * @brief pauseRecord 暂停录音
     */
    void slotPauseRecord();
    /**
     * @brief stopRecord 结束录音
     */
    void slotStopRecord();
    /**
     * @brief slotAudioBufferProbed
     * @param buffer
     */
    void slotAudioBufferProbed(const QAudioBuffer &buffer);

    /**
     * @brief slotDeviceChange 录音设备变化
     * @param mode
     */
    void slotDeviceChange(int mode);

private:
    void initRecordPath();
    void upDateWaveForm(const QAudioBuffer &buffer);
    QString formatRecordTime(qint64 millisecond);

private:
    qint64 m_recordMsec {0};
    QString m_recordTime {"00:00"};
    QString m_recordDir {""};
    QString m_recordName {""};
    GstreamRecorder *m_audioRecoder {nullptr};
    AudioWatcher *m_audioWatcher {nullptr};
    QList<qreal> m_audioScaleSamples;
    qreal m_maxSamples {0.01};
};

#endif // AUDIORECORDMANAGER_H
