/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "praseaudiowaveform.h"
#include "utils.h"

#include <QFileInfo>

PraseAudioWaveform::PraseAudioWaveform(QObject *parent)
    : QObject(parent)
{
    m_decoder = new QAudioDecoder(this);
    connect(m_decoder, &QAudioDecoder::bufferReady, this, &PraseAudioWaveform::slotSetBuffer);
    connect(m_decoder, &QAudioDecoder::finished, this, &PraseAudioWaveform::slotPraseDataEnd);
}

void PraseAudioWaveform::slotSetBuffer()
{
    m_buffer = m_decoder->read();
    qint64  current = m_remaining + m_buffer.sampleCount() / m_buffer.format().channelCount();
    if (current <= m_step) {
        m_remaining = current;
    } else {
        m_remaining = current % m_step;
        while (current >= m_step) {
            current -= m_step;
            qreal peak = Utils::getInstance()->getAudioWaveform(m_buffer, current);
            m_data.append(qAbs(peak));
        }
    }
}

void PraseAudioWaveform::slotPraseDataEnd()
{
    auto max = *(std::max_element(std::begin(m_data), std::end(m_data)));
    QList<qreal> data;
    for (int i = 0; i < m_data.size(); ++i) {
        qreal ft = max <= 0.0 ? 1.0 : m_data[i]  * m_maxWaveformHeight / max;
        data.push_back(qMax(ft, 1.0));
    }
    emit sigPraseFinsh(m_audoInfo.voicePath, data);
    this->deleteLater();
}

bool PraseAudioWaveform::startPrase()
{
    if (!m_maxSamples || !m_audoInfo.voiceSize) {
        return false;
    }
    qint64 dataCount =  m_audoInfo.voiceSize * 44100 / 1000;
    if (!dataCount) {
        this->deleteLater();
        return false;
    }
    m_step = dataCount / m_maxSamples;
    QFileInfo audio(m_audoInfo.voicePath);
    if (!audio.exists()) {
        this->deleteLater();
        return  false;
    }
    m_decoder->setSourceFilename(m_audoInfo.voicePath);
    m_decoder->start();
    return true;
}
